@@ -11,7 +11,7 @@ var TabmixTabClickOptions = {
 
     var target = aEvent.originalTarget;
     var anonid = target.getAttribute("anonid");
-    if (target == gBrowser.tabContainer.mTabsNewtabButton || (aEvent.button == 0 && anonid == "tmp-close-button"))
+    if (target == gBrowser.tabContainer.mTabsNewtabButton)
       this._blockButtonDblClick = true;
     else if (this._blockButtonDblClick)
       setTimeout(function() {TabmixTabClickOptions._blockButtonDblClick = false; }, 0);
@@ -28,7 +28,6 @@ var TabmixTabClickOptions = {
       return;
 
     this.onDoubleClick = false;
-
     var clickOutTabs = aEvent.target.localName == "tabs";
     var tab = clickOutTabs ? gBrowser.mCurrentTab : aEvent.target;
 
@@ -41,7 +40,7 @@ var TabmixTabClickOptions = {
     // for tab flip
     if (!clickOutTabs && aEvent.button == 0 && tab.hasAttribute("clickOnCurrent")) {
       tab.removeAttribute("clickOnCurrent");
-      let tabFlip = TabmixSvc.TMPprefs.getBoolPref("tabFlip");
+      let tabFlip = Tabmix.prefs.getBoolPref("tabFlip");
       if (tabFlip && !aEvent.shiftKey && !aEvent.ctrlKey && !aEvent.altKey && !aEvent.metaKey){
         let self = this;
         var selectPreviousTab = function (aTab) {
@@ -51,7 +50,7 @@ var TabmixTabClickOptions = {
             content.focus();
           }
         }
-        let tabFlipDelay = TabmixSvc.TMPprefs.getIntPref("tabFlipDelay");
+        let tabFlipDelay = Tabmix.prefs.getIntPref("tabFlipDelay");
         setTimeout(function (aTab) {selectPreviousTab(aTab);}, tabFlipDelay, tab);
         return;
       }
@@ -93,7 +92,7 @@ var TabmixTabClickOptions = {
     this.onDoubleClick = true;
     var node = aEvent.originalTarget;
 
-    var blockDblClick = Tabmix.isVersion(40) ? gBrowser.tabContainer._blockDblClick : gBrowser._blockDblClick
+    var blockDblClick = gBrowser.tabContainer._blockDblClick;
     if (blockDblClick && aEvent.originalTarget.localName == "box")
       return;
     // don't do anything if user click on close tab button , or on any other button on tab or tabbar
@@ -153,7 +152,7 @@ var TabmixTabClickOptions = {
         TMP_ClosedTabs.undoCloseTab();
         break;
       case 11:
-        gBrowser.renameTab(aTab);
+        Tabmix.renameTab.editTitle(aTab);
         break;
       case 12: //taken from tco
         if (SessionSaver && SessionSaver.snapBackTab)
@@ -240,12 +239,10 @@ var TabmixTabClickOptions = {
         Tabmix.autoReload.toggle(aTab);
         break;
       case 31: // pin/unpin tab
-        if (Tabmix.isVersion(40)) {
-          if (aTab.pinned)
-            gBrowser.unpinTab(aTab);
-          else
-            gBrowser.pinTab(aTab);
-        }
+        if (aTab.pinned)
+          gBrowser.unpinTab(aTab);
+        else
+          gBrowser.pinTab(aTab);
         break;
     }
   }
@@ -256,44 +253,22 @@ var TabmixContext = {
   buildTabContextMenu: function TMP_buildTabContextMenu() {
     var tabContextMenu;
     var closeOther = document.getElementById("context_closeOtherTabs");
-    if (Tabmix.isVersion(40)) {
-      tabContextMenu = gBrowser.tabContextMenu;
-      let reloadAll = document.getElementById("context_reloadAllTabs");
-      // move context_reloadAllTabs to the place it was before Fireofox 4.0
-      tabContextMenu.insertBefore(reloadAll, document.getElementById("context_reloadTab").nextSibling);
-      // move context_closeOtherTabs to the place it was before Fireofox 4.0
-      tabContextMenu.insertBefore(closeOther, reloadAll.nextSibling);
-      // from Firefox 4.0 2009-09-11 there is gContextMenu.openLinkInCurrent in contentAreaContextMenu
-      // move our menu to that position
-      let openLinkInCurrent = document.getElementById("context-openlinkincurrent");
-      let tabmix_menu = document.getElementById("tm-openlinkhere");
-      openLinkInCurrent.parentNode.insertBefore(tabmix_menu, openLinkInCurrent);
-
-      if (!document.getElementById("context_bookmarkTab")) {
-        // in firefox 4.0 context_bookmarkTab was removed
-        let bookmarkTab = document.createElement("menuitem");
-        tabContextMenu.insertBefore(bookmarkTab, document.getElementById("context_bookmarkAllTabs"));
-        bookmarkTab.setAttribute("id", "context_bookmarkTab");
-        let label = document.getElementById("tm-separator").getAttribute("_bookmarkTab");
-        bookmarkTab.setAttribute("label", label);
-        bookmarkTab.setAttribute("oncommand", "PlacesCommandHook.bookmarkPage(TabContextMenu.contextTab.linkedBrowser, PlacesUtils.bookmarksMenuFolderId, true);");
-      }
+    tabContextMenu = gBrowser.tabContextMenu;
+    let reloadAll = document.getElementById("context_reloadAllTabs");
+    // move context_reloadAllTabs to the place it was before Fireofox 4.0
+    tabContextMenu.insertBefore(reloadAll, document.getElementById("context_reloadTab").nextSibling);
+    // move context_closeOtherTabs to the place it was before Fireofox 4.0
+    tabContextMenu.insertBefore(closeOther, reloadAll.nextSibling);
+    if (!document.getElementById("context_bookmarkTab")) {
+      // in firefox 4.0 context_bookmarkTab was removed
+      let bookmarkTab = document.createElement("menuitem");
+      tabContextMenu.insertBefore(bookmarkTab, document.getElementById("context_bookmarkAllTabs"));
+      bookmarkTab.setAttribute("id", "context_bookmarkTab");
+      let label = document.getElementById("tm-separator").getAttribute("_bookmarkTab");
+      bookmarkTab.setAttribute("label", label);
+      bookmarkTab.setAttribute("oncommand", "PlacesCommandHook.bookmarkPage(TabContextMenu.contextTab.linkedBrowser, PlacesUtils.bookmarksMenuFolderId, true);");
     }
-    else {
-      if (!gBrowser.tabContextMenu)
-        gBrowser.tabContextMenu = document.getAnonymousElementByAttribute(gBrowser, "anonid", "tabContextMenu");
-      tabContextMenu = gBrowser.tabContextMenu;
 
-      if (!tabContextMenu.hasAttribute("id"))
-        tabContextMenu.setAttribute("id", "MenuEdit" in window ? "menuedit-tabContextMenu" : "tabContextMenu");
-
-      // we use it to make sure context menu is not hide when stip is collapsed in Firefox 3.5-3.6
-      let _class = tabContextMenu.hasAttribute("class") ? tabContextMenu.setAttribute("class") + " ": "";
-      tabContextMenu.setAttribute("class", _class + "tabmix-tabContextMenu");
-      // From Firefox 3.5.4pre we can not use anonymous menupopup from toolbar button
-      // this hack enable tabmix to use tab context menu from tablist tollbar button
-      gBrowser.tabContainer.parentNode.insertBefore(gBrowser.tabContextMenu, gBrowser.tabContainer);
-    }
     // to be compatible with firefox 3.0 we use our own move-tab-to-new-window
     // firefox don't have function to duplicate the tab to new windows
     // look in gBrowser.duplicateInWindow
@@ -302,12 +277,12 @@ var TabmixContext = {
     if (item) // make sure no one removed it before we do
       tabContextMenu.removeChild(item);
 
-    tabContextMenu.setAttribute("onpopuphidden", tabContextMenu.getAttribute("onpopuphidden") + "if (event.target == this) TMP_hidePopup();");
+    tabContextMenu.setAttribute("onpopuphidden", tabContextMenu.getAttribute("onpopuphidden") + "if (event.target == this) Tabmix.hidePopup(this);");
     tabContextMenu.addEventListener("popupshowing", this.updateTabContextMenu, false);
     tabContextMenu.addEventListener("popupshown", this.tabContextMenuShown, false);
 
     // update context_newTab attribute
-    if (Tabmix.isVersion(40) && !document.getElementById("context_newTab")) {
+    if (!document.getElementById("context_newTab")) {
       // in firefox 4.0 context_newTab was removed
       // we insert context_newTab back as firstChild
       let firstChild = tabContextMenu.firstChild;
@@ -322,15 +297,8 @@ var TabmixContext = {
       newTab.setAttribute("tmp_iconic", "menuitem-iconic newtab-menu-icon");
       newTab.setAttribute("oncommand", "BrowserOpenTab();");
     }
-    else {
-      let newTab = document.getElementById("context_newTab");
-      newTab.setAttribute("_newtab", newTab.getAttribute("label"));
-      newTab.setAttribute("_afterthis", document.getElementById("tm-separator").getAttribute("_afterthis"));
-      newTab.setAttribute("tmp_iconic", "menuitem-iconic newtab-menu-icon");
-      newTab.removeAttribute(Tabmix.isVersion(40) ? "command" : "xbl:inherits");
-    }
 
-    var separators = tabContextMenu.getElementsByTagName(Tabmix.isVersion(40) ? "menuseparator" : "xul:menuseparator");
+    var separators = tabContextMenu.getElementsByTagName("menuseparator");
     for (let i = 0; i < separators.length; i++ ) {
       let separator = separators[i];
       if (separator.parentNode !=  tabContextMenu || separator.hasAttribute("id"))
@@ -369,14 +337,12 @@ var TabmixContext = {
     openTabInWindow.setAttribute("id", "context_openTabInWindow");
 
     // Move pin/unpin and tabViewMenu after context_openTabInWindow
-    if (Tabmix.isVersion(40)) {
-      try { // temp for a few weeks 2010-07-07
+    try { // temp for a few weeks 2010-07-07
       let item = openTabInWindow.nextSibling;
-       tabContextMenu.insertBefore(document.getElementById("context_pinTab"), item);
-       tabContextMenu.insertBefore(document.getElementById("context_unpinTab"), item);
-       tabContextMenu.insertBefore(document.getElementById("context_tabViewMenu"), item);
-      } catch (ex) { }
-    }
+      tabContextMenu.insertBefore(document.getElementById("context_pinTab"), item);
+      tabContextMenu.insertBefore(document.getElementById("context_unpinTab"), item);
+      tabContextMenu.insertBefore(document.getElementById("context_tabViewMenu"), item);
+    } catch (ex) { }
 
     // move context_undoCloseTab before tm-undoCloseList
     var _undoCloseTab = document.getElementById("context_undoCloseTab");
@@ -403,95 +369,88 @@ var TabmixContext = {
   },
 
   // Tab context menu popupshowing
-//Tabmix.updateTabContextMenu = function TMP_updateTabContextMenu(event) {
   updateTabContextMenu: function TMP_updateTabContextMenu(event) {
     // 'this' here refer to tabContextMenu
 //XXX test if we need this on FF 3.5+
     // Undo close tab Commands - firefox 3.0 toggle this menu item also when submneu is show
-    var undoClose = TabmixSvc.TMPprefs.getBoolPref("undoClose");
-    Tabmix.showItem("context_undoCloseTab", TabmixSvc.TMPprefs.getBoolPref("undoCloseTabMenu") && undoClose);
+    var undoClose = Tabmix.prefs.getBoolPref("undoClose");
+    Tabmix.showItem("context_undoCloseTab", Tabmix.prefs.getBoolPref("undoCloseTabMenu") && undoClose);
 
     if (event.originalTarget != gBrowser.tabContextMenu)
       return true;
 
-    var item;
-    if (document.popupNode.parentNode)
-      item = document.popupNode.parentNode.parentNode.id;
-    if (item && (item == "btn_tabslist" || item == "btn_tabslistSorted" || item == "alltabs-button")) {
-      if (Tabmix.isVersion(40))
-        TabContextMenu.contextTab = document.popupNode.tab;
-      else
-        gBrowser.mContextTab = document.popupNode.tab;
-    }
+    var item, triggerNode = gBrowser.tabContextMenu.triggerNode;
+    if (triggerNode.parentNode)
+      item = triggerNode.parentNode.id;
+    if (item && (item == "btn_tabslist_menu" || item == "alltabs-popup"))
+      TabContextMenu.contextTab = triggerNode.tab;
 
-    var clickOutTabs = document.popupNode.localName == "tabs";
-    var aTab = clickOutTabs ? gBrowser.mCurrentTab : gBrowser.mContextTab;
+    var clickOutTabs = triggerNode.localName == "tabs";
+    var aTab = clickOutTabs ? gBrowser.selectedTab : TabContextMenu.contextTab;
 
     var isOneWindow = Tabmix.numberOfWindows() == 1;
 
     var newTab = document.getElementById("context_newTab");
-    Tabmix.showItem(newTab, TabmixSvc.TMPprefs.getBoolPref("newTabMenu"));
+    Tabmix.showItem(newTab, Tabmix.prefs.getBoolPref("newTabMenu"));
     if (clickOutTabs) {
       Tabmix.setItem(newTab, "label", newTab.getAttribute("_newtab"));
       Tabmix.setItem(newTab, "oncommand", "TMP_BrowserOpenTab();");
     }
     else {
       Tabmix.setItem(newTab, "label", newTab.getAttribute("_newtab") + "  " + newTab.getAttribute("_afterthis"));
-      Tabmix.setItem(newTab, "oncommand", "TMP_BrowserOpenTab(gBrowser.mContextTab);");
+      Tabmix.setItem(newTab, "oncommand", "TMP_BrowserOpenTab(TabContextMenu.contextTab);");
     }
 
     // Duplicate Commands
-    Tabmix.showItem("tm-duplicateTab", TabmixSvc.TMPprefs.getBoolPref("duplicateMenu"));
-    Tabmix.showItem("tm-duplicateinWin", TabmixSvc.TMPprefs.getBoolPref("duplicateinWinMenu") && !Tabmix.singleWindowMode);
-    Tabmix.showItem("context_openTabInWindow", TabmixSvc.TMPprefs.getBoolPref("detachTabMenu") && !Tabmix.singleWindowMode);
-    if (Tabmix.isVersion(40)) {
-      let show = TabmixSvc.TMPprefs.getBoolPref("pinTabMenu");
-      Tabmix.showItem("context_pinTab", show && !aTab.pinned);
-      Tabmix.showItem("context_unpinTab", show && aTab.pinned);
-      // context_tabViewMenu exist from Firefox 4.0b4
-      Tabmix.showItem("context_tabViewMenu", TabmixSvc.TMPprefs.getBoolPref("moveToGroup") && !aTab.pinned);
-    }
-    Tabmix.showItem("tm-mergeWindowsTab", TabmixSvc.TMPprefs.getBoolPref("showMergeWindow") && (!Tabmix.singleWindowMode || (Tabmix.singleWindowMode && !isOneWindow)));
-    var showRenameTabMenu = TabmixSvc.TMPprefs.getBoolPref("renameTabMenu");
+    Tabmix.showItem("tm-duplicateTab", Tabmix.prefs.getBoolPref("duplicateMenu"));
+    Tabmix.showItem("tm-duplicateinWin", Tabmix.prefs.getBoolPref("duplicateinWinMenu") && !Tabmix.singleWindowMode);
+    Tabmix.showItem("context_openTabInWindow", Tabmix.prefs.getBoolPref("detachTabMenu") && !Tabmix.singleWindowMode);
+    var show = Tabmix.prefs.getBoolPref("pinTabMenu");
+    Tabmix.showItem("context_pinTab", show && !aTab.pinned);
+    Tabmix.showItem("context_unpinTab", show && aTab.pinned);
+    // context_tabViewMenu exist from Firefox 4.0b4
+    Tabmix.showItem("context_tabViewMenu", Tabmix.prefs.getBoolPref("moveToGroup") && !aTab.pinned);
+    Tabmix.showItem("tm-mergeWindowsTab", Tabmix.prefs.getBoolPref("showMergeWindow") && (!Tabmix.singleWindowMode || (Tabmix.singleWindowMode && !isOneWindow)));
+    var showRenameTabMenu = Tabmix.prefs.getBoolPref("renameTabMenu");
     Tabmix.showItem("tm-renameTab", showRenameTabMenu);
-    Tabmix.showItem("tm-copyTabUrl", TabmixSvc.TMPprefs.getBoolPref("copyTabUrlMenu"));
+    Tabmix.showItem("tm-copyTabUrl", Tabmix.prefs.getBoolPref("copyTabUrlMenu"));
 
     //  ---------------- menuseparator ---------------- //
 
     // Reload Commands
-    Tabmix.showItem("context_reloadTab", TabmixSvc.TMPprefs.getBoolPref("reloadTabMenu"));
-    Tabmix.showItem("context_reloadAllTabs", TabmixSvc.TMPprefs.getBoolPref("reloadAllMenu"));
-    Tabmix.showItem("tm-autoreloadTab_menu", TabmixSvc.TMPprefs.getBoolPref("autoReloadMenu"));
-    Tabmix.showItem("tm-reloadRight", TabmixSvc.TMPprefs.getBoolPref("reloadRightMenu"));
-    Tabmix.showItem("tm-reloadLeft", TabmixSvc.TMPprefs.getBoolPref("reloadLeftMenu"));
-    Tabmix.showItem("tm-reloadOther", TabmixSvc.TMPprefs.getBoolPref("reloadOtherMenu"));
+    Tabmix.showItem("context_reloadTab", Tabmix.prefs.getBoolPref("reloadTabMenu"));
+    Tabmix.showItem("context_reloadAllTabs", Tabmix.prefs.getBoolPref("reloadAllMenu"));
+    Tabmix.showItem("tm-autoreloadTab_menu", Tabmix.prefs.getBoolPref("autoReloadMenu"));
+    Tabmix.showItem("tm-reloadRight", Tabmix.prefs.getBoolPref("reloadRightMenu"));
+    Tabmix.showItem("tm-reloadLeft", Tabmix.prefs.getBoolPref("reloadLeftMenu"));
+    Tabmix.showItem("tm-reloadOther", Tabmix.prefs.getBoolPref("reloadOtherMenu"));
 
     //  ---------------- menuseparator ---------------- //
 
-    Tabmix.showItem("tm-undoCloseList", TabmixSvc.TMPprefs.getBoolPref("undoCloseListMenu") && undoClose);
+    Tabmix.showItem("tm-undoCloseList", Tabmix.prefs.getBoolPref("undoCloseListMenu") && undoClose);
 
     //  ---------------- menuseparator ---------------- //
 
     // Close tab Commands
-    var pinnedTab = Tabmix.isVersion(40) && TabContextMenu.contextTab.pinned;
-    Tabmix.showItem("context_closeTab", TabmixSvc.TMPprefs.getBoolPref("closeTabMenu"));
-    Tabmix.showItem("tm-closeAllTabs", TabmixSvc.TMPprefs.getBoolPref("closeAllMenu") && !pinnedTab);
-    Tabmix.showItem("tm-closeSimilar", TabmixSvc.TMPprefs.getBoolPref("closeSimilarTabs") && !pinnedTab);
-    Tabmix.showItem("context_closeOtherTabs", TabmixSvc.TMPprefs.getBoolPref("closeOtherMenu") && !pinnedTab);
-    Tabmix.showItem("tm-closeLeftTabs", TabmixSvc.TMPprefs.getBoolPref("closeLeftMenu") && !pinnedTab);
-    Tabmix.showItem("tm-closeRightTabs", TabmixSvc.TMPprefs.getBoolPref("closeRightMenu") && !pinnedTab);
+    var pinnedTab = TabContextMenu.contextTab.pinned;
+    Tabmix.showItem("context_closeTab", Tabmix.prefs.getBoolPref("closeTabMenu"));
+    Tabmix.showItem("tm-closeAllTabs", Tabmix.prefs.getBoolPref("closeAllMenu") && !pinnedTab);
+    Tabmix.showItem("tm-closeSimilar", Tabmix.prefs.getBoolPref("closeSimilarTabs") && !pinnedTab);
+    Tabmix.showItem("context_closeOtherTabs", Tabmix.prefs.getBoolPref("closeOtherMenu") && !pinnedTab);
+    Tabmix.showItem("tm-closeLeftTabs", Tabmix.prefs.getBoolPref("closeLeftMenu") && !pinnedTab);
+    Tabmix.showItem("tm-closeRightTabs", Tabmix.prefs.getBoolPref("closeRightMenu") && !pinnedTab);
 
     //  ---------------- menuseparator ---------------- //
 
-    Tabmix.showItem("tm-docShell", TabmixSvc.TMPprefs.getBoolPref("docShellMenu"));
-    Tabmix.showItem("tm-freezeTab", TabmixSvc.TMPprefs.getBoolPref("freezeTabMenu"));
-    Tabmix.showItem("tm-protectTab", TabmixSvc.TMPprefs.getBoolPref("protectTabMenu"));
-    Tabmix.showItem("tm-lockTab", TabmixSvc.TMPprefs.getBoolPref("lockTabMenu"));
+    Tabmix.showItem("tm-docShell", Tabmix.prefs.getBoolPref("docShellMenu"));
+    Tabmix.showItem("tm-freezeTab", Tabmix.prefs.getBoolPref("freezeTabMenu"));
+    Tabmix.showItem("tm-protectTab", Tabmix.prefs.getBoolPref("protectTabMenu"));
+    Tabmix.showItem("tm-lockTab", Tabmix.prefs.getBoolPref("lockTabMenu"));
 
     //  ---------------- menuseparator ---------------- //
 
-    Tabmix.showItem("context_bookmarkTab", TabmixSvc.TMPprefs.getBoolPref("bookmarkTabMenu"));
-    Tabmix.showItem("context_bookmarkAllTabs", TabmixSvc.TMPprefs.getBoolPref("bookmarkTabsMenu"));
+    Tabmix.showItem("context_bookmarkTab", Tabmix.prefs.getBoolPref("bookmarkTabMenu"));
+    Tabmix.showItem("context_bookmarkAllTabs", Tabmix.prefs.getBoolPref("bookmarkTabsMenu"));
 
     // we call this again when popupshown to make sure we don't show 2 menuseparator together
     TabmixContext.tabContextMenuShown(event);
@@ -501,9 +460,9 @@ var TabmixContext = {
       let titleNotReady;
       if (aTab.hasAttribute("busy")) {
         let browser = gBrowser.getBrowserForTab(aTab);
-        let url = browser.contentDocument.baseURI || browser.currentURI.spec;
-        let docTitle = TMP_getTitleFromBookmark(url, browser.contentDocument.title, aTab.getAttribute("tabmix_bookmarkId"));
-        if (!docTitle || docTitle == gBrowser.mStringBundle.getString(Tabmix.isVersion(40) ? "tabs.emptyTabTitle" : "tabs.untitled"))
+        let url = browser.currentURI.spec;
+        let docTitle = TMP_Places.getTitleFromBookmark(url, browser.contentDocument.title, null, aTab);
+        if (!docTitle || docTitle == gBrowser.mStringBundle.getString("tabs.emptyTabTitle"))
           titleNotReady = true;
       }
       Tabmix.setItem("tm-renameTab", "disabled", titleNotReady);
@@ -511,20 +470,13 @@ var TabmixContext = {
 
     var protectedTab = aTab.hasAttribute("protected");
     var lockedTab = aTab.hasAttribute("locked");
-    var tabsCount, unpinnedTabs, cIndex;
-    if (Tabmix.isVersion(40)) {
-      tabsCount = gBrowser.visibleTabs.length;
-      unpinnedTabs = tabsCount - gBrowser.tabContainer._real_numPinnedTabs;
-      cIndex = TMP_TabView.getIndexInVisibleTabsFromTab(aTab);
-    }
-    else {
-      tabsCount = unpinnedTabs = gBrowser.tabs.length;
-      cIndex = aTab._tPos;
-    }
+    var tabsCount = gBrowser.visibleTabs.length;
+    var unpinnedTabs = tabsCount - gBrowser.tabContainer._real_numPinnedTabs;
+    var cIndex = TMP_TabView.getIndexInVisibleTabsFromTab(aTab);
     if (Tabmix.rtl)
       cIndex = tabsCount - 1 - cIndex;
 
-    var keepLastTab = tabsCount == 1 && TabmixSvc.TMPprefs.getBoolPref("keepLastTab");
+    var keepLastTab = tabsCount == 1 && Tabmix.prefs.getBoolPref("keepLastTab");
     Tabmix.setItem("context_closeTab", "disabled", protectedTab || keepLastTab);
     Tabmix.setItem("tm-closeAllTabs", "disabled", keepLastTab || unpinnedTabs <= 1);
     Tabmix.setItem("context_closeOtherTabs", "disabled", unpinnedTabs <= 1);
@@ -621,73 +573,66 @@ var TabmixContext = {
       * from Firefox 4.0 2009-09-11 there is gContextMenu.openLinkInCurrent
       * Firefox only show this menu when the selection text is url see Bug 454518
       * we cahck if gContextMenu.linkURL contain URL
-
-      * we can remove this menu in tabmix.js but it is safer to just hide it each time
       */
-      if (Tabmix.isVersion(40)) {
-        // the label for this menu is "Open Link"
-        // we use our own menu "Open Link in This Tab"
-        Tabmix.showItem("context-openlinkincurrent", false);
-      }
       var onLink = gContextMenu.onLink || gContextMenu.linkURL;
-      Tabmix.showItem("tm-openlinkhere", TabmixSvc.TMPprefs.getBoolPref("openLinkHere") && onLink);
+      Tabmix.showItem("context-openlinkincurrent", Tabmix.prefs.getBoolPref("openLinkHere") && onLink);
       var inverseLink = document.getElementById("tm-openinverselink");
-      Tabmix.showItem(inverseLink, TabmixSvc.TMPprefs.getBoolPref("openInverseLink") && onLink);
+      Tabmix.showItem(inverseLink, Tabmix.prefs.getBoolPref("openInverseLink") && onLink);
       if (!inverseLink.hidden){
-        let bgPref = TabmixSvc.prefs.getBoolPref("browser.tabs.loadInBackground");
+        let bgPref = Services.prefs.getBoolPref("browser.tabs.loadInBackground");
         let focusType = bgPref ? "fg":"bg";
         inverseLink.setAttribute("label", inverseLink.getAttribute(focusType+"label"));
         inverseLink.setAttribute("accesskey", inverseLink.getAttribute(focusType+"accesskey"));
       }
-      Tabmix.showItem("tm-linkWithhistory", TabmixSvc.TMPprefs.getBoolPref("linkWithHistory") && onLink);
+      Tabmix.showItem("tm-linkWithhistory", Tabmix.prefs.getBoolPref("linkWithHistory") && onLink);
       var closeTabMenu = document.getElementById("tm-content-closetab");
-      Tabmix.showItem(closeTabMenu, !contentClick && TabmixSvc.TMPprefs.getBoolPref("closeTabContent"));
-      var keepLastTab = tabsCount == 1 && TabmixSvc.TMPprefs.getBoolPref("keepLastTab");
+      Tabmix.showItem(closeTabMenu, !contentClick && Tabmix.prefs.getBoolPref("closeTabContent"));
+      var keepLastTab = tabsCount == 1 && Tabmix.prefs.getBoolPref("keepLastTab");
       Tabmix.setItem(closeTabMenu, "disabled", protectedTab || keepLastTab);
 
       var freezeTabMenu = document.getElementById("tm-content-freezeTab");
-      Tabmix.showItem(freezeTabMenu, !contentClick && TabmixSvc.TMPprefs.getBoolPref("freezeTabContent"));
+      Tabmix.showItem(freezeTabMenu, !contentClick && Tabmix.prefs.getBoolPref("freezeTabContent"));
       Tabmix.setItem(freezeTabMenu, "checked", protectedTab && lockedTab);
 
       var lockTabMenu = document.getElementById("tm-content-lockTab");
-      Tabmix.showItem(lockTabMenu, !contentClick && TabmixSvc.TMPprefs.getBoolPref("lockTabContent"));
+      Tabmix.showItem(lockTabMenu, !contentClick && Tabmix.prefs.getBoolPref("lockTabContent"));
       Tabmix.setItem(lockTabMenu, "checked", lockedTab);
 
       var protectTabMenu = document.getElementById("tm-content-protectTab");
-      Tabmix.showItem(protectTabMenu, !contentClick && TabmixSvc.TMPprefs.getBoolPref("protectTabContent"));
+      Tabmix.showItem(protectTabMenu, !contentClick && Tabmix.prefs.getBoolPref("protectTabContent"));
       Tabmix.setItem(protectTabMenu, "checked", protectedTab);
 
       var duplicateTabMenu = document.getElementById("tm-duplicateTabContext");
       Tabmix.showItem(duplicateTabMenu, !contentClick && !gContextMenu.isTextSelected &&
-                     TabmixSvc.TMPprefs.getBoolPref("duplicateTabContent"));
+                     Tabmix.prefs.getBoolPref("duplicateTabContent"));
 
       Tabmix.showItem("tm-detachTabContext", !contentClick && !gContextMenu.isTextSelected && !Tabmix.singleWindowMode &&
-                     TabmixSvc.TMPprefs.getBoolPref("detachTabContent"));
+                     Tabmix.prefs.getBoolPref("detachTabContent"));
 
       var duplicateWinMenu = document.getElementById("tm-duplicateinWinContext");
       Tabmix.showItem(duplicateWinMenu, !contentClick && !gContextMenu.isTextSelected && !Tabmix.singleWindowMode &&
-                     TabmixSvc.TMPprefs.getBoolPref("duplicateWinContent"));
+                     Tabmix.prefs.getBoolPref("duplicateWinContent"));
 
       var tabsListMenu = document.getElementById("tm-tabsList");
-      Tabmix.showItem(tabsListMenu, !contentClick && TabmixSvc.TMPprefs.getBoolPref("tabsList"));
+      Tabmix.showItem(tabsListMenu, !contentClick && Tabmix.prefs.getBoolPref("tabsList"));
 
       var undoCloseTabMenu = document.getElementById("tm-content-undoCloseTab");
-      var undoClose = TabmixSvc.TMPprefs.getBoolPref("undoClose");
+      var undoClose = Tabmix.prefs.getBoolPref("undoClose");
       Tabmix.showItem(undoCloseTabMenu, !contentClick && !gContextMenu.isTextSelected && undoClose && !closeTabsEmpty &&
-                     TabmixSvc.TMPprefs.getBoolPref("undoCloseTabContent"));
+                     Tabmix.prefs.getBoolPref("undoCloseTabContent"));
 
       var undoCloseListMenu = document.getElementById("tm-content-undoCloseList");
       Tabmix.showItem(undoCloseListMenu, !contentClick && !gContextMenu.isTextSelected && undoClose && !closeTabsEmpty &&
-                     TabmixSvc.TMPprefs.getBoolPref("undoCloseListContent"));
+                     Tabmix.prefs.getBoolPref("undoCloseListContent"));
 
       var isOneWindow = Tabmix.numberOfWindows() == 1;
       var mergeMenu = document.getElementById("tm-mergeWindows");
-      Tabmix.showItem(mergeMenu, !contentClick && !isOneWindow && TabmixSvc.TMPprefs.getBoolPref("mergeWindowContent"));
+      Tabmix.showItem(mergeMenu, !contentClick && !isOneWindow && Tabmix.prefs.getBoolPref("mergeWindowContent"));
 
       Tabmix.showItem("tm-autoreload_menu", !contentClick && !gContextMenu.isTextSelected &&
-                     TabmixSvc.TMPprefs.getBoolPref("autoReloadContent"));
+                     Tabmix.prefs.getBoolPref("autoReloadContent"));
 
-      Tabmix.showItem("tm-openAllLinks", TabmixSvc.TMPprefs.getBoolPref("openAllLinks") && !TabmixContext.openMultipleLinks(true));
+      Tabmix.showItem("tm-openAllLinks", Tabmix.prefs.getBoolPref("openAllLinks") && !TabmixContext.openMultipleLinks(true));
 
       // show/hide menuseparator
       var undoCloseSep = document.getElementById("tm-content-undoCloseSep");
@@ -760,32 +705,21 @@ var TabmixContext = {
       nextEpisode = treeWalker.nextNode();
     }
     if (urls.length)
-     TMP_loadTabs(urls, false);
+     Tabmix.loadTabs(urls, false);
     return true;
   }
 }
 
-// sometimes context popup stay "open", we hide it manually.
-//function TM_hidePopup () {
-function TMP_hidePopup() {
-   var node = document.popupNode;
-   while (node && node.localName != "menubar" && node.localName != "toolbar") {
-      if (node.localName == "menupopup" || node.localName == "popup") {
-         if (node.hasAttribute("open")) node.removeAttribute("open");
-         node.hidePopup();
-      }
-      node = node.parentNode;
-   }
-}
-
 // for all tabs popup lists
 var TabmixAllTabs = {
   _selectedItem: null,
+  _popup: null,
   backupLabel: "",
   handleEvent: function TMP_AT_handleEvent(aEvent) {
     switch (aEvent.type) {
-      case this._attrModifiedEvent:
-        this._tabOnAttrModified(aEvent);
+      case "TabAttrModified":
+        let tab = aEvent.target;
+        this._setMenuitemAttributes(tab.mCorrespondingMenuitem, tab);
         break;
       case "TabClose":
         this._tabOnTabClose(aEvent);
@@ -798,7 +732,33 @@ var TabmixAllTabs = {
         break;
       case "DOMMouseScroll":
         TMP_eventListener.onTabBarScroll(aEvent);
+      case "scroll":
+        this._popup._updateTabsVisibilityStatus();
         break;
+      case "popupshown":
+        this._ensureElementIsVisible(aEvent);
+        break;
+    }
+  },
+
+  _updateTabsVisibilityStatus: function TMP__updateTabsVisibilityStatus() {
+    var tabContainer = gBrowser.tabContainer;
+    // We don't want menu item decoration unless there is overflow.
+    if (tabContainer.getAttribute("overflow") != "true")
+      return;
+
+    var tabstripBO = tabContainer.mTabstrip.scrollBoxObject;
+    for (var i = 0; i < this.childNodes.length; i++) {
+      let curTab = this.childNodes[i].tab;
+      if (!curTab) // "Tab Groups" menuitem and its menuseparator
+        continue;
+      let curTabBO = curTab.boxObject;
+      if (!curTabBO) // "Tabs From Other Computers" menuitem
+        continue;
+      if (tabContainer.mTabstrip.isElementVisible(curTab))
+        this.childNodes[i].setAttribute("tabIsVisible", "true");
+      else
+        this.childNodes[i].removeAttribute("tabIsVisible");
     }
   },
 
@@ -838,7 +798,7 @@ var TabmixAllTabs = {
   },
 
   removeTabFromList: function TMP_removeTabFromList(event, popup, aType) {
-    if (!TabmixSvc.TMPprefs.getBoolPref("middleclickDelete"))
+    if (!Tabmix.prefs.getBoolPref("middleclickDelete"))
       return;
 
     if (event.target.id == "btn_tabslistSorted")
@@ -879,16 +839,19 @@ var TabmixAllTabs = {
     if (popup.hasAttribute("context") && popup.getAttribute("context") != tabContextMenu.id)
       popup.setAttribute("context", tabContextMenu.id);
 
-    if (TabmixSvc.prefs.getBoolPref("extensions.tabmix.enableScrollSwitch"))
-      popup.addEventListener("DOMMouseScroll", this, true);
-
     this.beforeCommonList(popup);
     this.createCommonList(popup, aType);
 
+    gBrowser.tabContainer.mTabstrip.addEventListener("scroll", this, false);
+    this._popup = popup;
+    if (!this._popup._updateTabsVisibilityStatus)
+      this._popup._updateTabsVisibilityStatus = this._updateTabsVisibilityStatus;
+    this._popup._updateTabsVisibilityStatus();
+
     return true;
   },
 
-  beforeCommonList: function TMP_beforeCommonList(popup) {
+  beforeCommonList: function TMP_beforeCommonList(popup, aCloseTabsPopup) {
     var item = popup.parentNode;
     if (item.id == "btn_tabslist" || item.id == "btn_undoclose")
       item.removeAttribute("tooltiptext");
@@ -902,14 +865,10 @@ var TabmixAllTabs = {
       popup.removeChild(menuItem);
     }
 
-    if (Tabmix.isVersion(40)) {
-      this._attrModifiedEvent = "TabAttrModified";
+    if (!aCloseTabsPopup) {
       gBrowser.tabContainer.addEventListener("TabAttrModified", this, false);
+      gBrowser.tabContainer.addEventListener("TabClose", this, false);
     }
-    else
-      this._attrModifiedEvent = "DOMAttrModified";
-
-    gBrowser.tabContainer.addEventListener("TabClose", this, false);
     popup.addEventListener("DOMMenuItemActive", this, false);
     popup.addEventListener("DOMMenuItemInactive", this, false);
   },
@@ -925,7 +884,7 @@ var TabmixAllTabs = {
           this.Index = index;
         }
         _tabSorting.prototype.toString = function() { return this.Tab.label.toLowerCase(); }
-        let visibleTabs = gBrowser.tabs;
+        let visibleTabs = gBrowser.visibleTabs;
         tabs = new Array(visibleTabs.length);
         for (i = 0; i < visibleTabs.length; i++)
           tabs[i] = new _tabSorting(visibleTabs[i], i);
@@ -934,11 +893,11 @@ var TabmixAllTabs = {
           this.createMenuItems(popup, tabs[i].Tab, tabs[i].Index, aType);
         break;
       case 2:
-        tabs = gBrowser.tabs;
+        tabs = gBrowser.visibleTabs;
         let addToMenu = side != "right";
         for (let t = 0; t < tabs.length; t++) {
           let tab = tabs[t];
-          let visible = side && gBrowser.tabContainer.isTabVisible(tab._tPos);
+          let visible = side && gBrowser.tabContainer.mTabstrip.isElementVisible(tab);
           if (visible) {
             if (tab.pinned)
               continue;
@@ -954,22 +913,25 @@ var TabmixAllTabs = {
       case 3:
         for (i = TMP_LastTab.tabs.length - 1; i >= 0; i--) {
           let tab = TMP_LastTab.tabs[i];
+          if (tab.hidden)
+            continue;
           this.createMenuItems(popup, tab, i, aType);
         }
         break;
     }
 
-    if (this._selectedItem) {
-      let scrollBox = document.getAnonymousElementByAttribute(popup, "class", "popup-internal-box");
-      if (!popup.style.getPropertyValue("max-height")) {
-        popup.style.setProperty("max-height",Math.round(screen.availHeight * 0.7) + "px", "important");
-        scrollBox.ensureElementIsVisible(popup.firstChild);
-      }
-      let items = Array.slice(popup.childNodes);
-      let element = items.indexOf(this._selectedItem) < popup.childElementCount/2 ? popup.firstChild : popup.lastChild;
-      scrollBox.ensureElementIsVisible(element);
-      scrollBox.ensureElementIsVisible(this._selectedItem);
-    }
+    if (this._selectedItem)
+      popup.addEventListener("popupshown", this, false);
+  },
+
+  _ensureElementIsVisible: function TMP__ensureElementIsVisible(event) {
+    var popup = event.target;
+    popup.removeEventListener("popupshown", this, false);
+    let scrollBox = document.getAnonymousElementByAttribute(popup, "class", "popup-internal-box");
+    let items = Array.slice(popup.childNodes);
+    let element = items.indexOf(this._selectedItem) < popup.childElementCount/2 ? popup.firstChild : popup.lastChild;
+    scrollBox.ensureElementIsVisible(element);
+    scrollBox.ensureElementIsVisible(this._selectedItem);
   },
 
   createMenuItems: function TMP_createMenuItems(popup, tab, value, aType) {
@@ -989,8 +951,6 @@ var TabmixAllTabs = {
 
     mi.value = value;
     tab.mCorrespondingMenuitem = mi;
-    if (!Tabmix.isVersion(40))
-      tab.addEventListener("DOMAttrModified", this, false);
     mi.tab = tab;
 
     if (popup.id == "btn_tabslist_menu")
@@ -1009,32 +969,6 @@ var TabmixAllTabs = {
     }
   },
 
-  _tabOnAttrModified: function TMP__tabOnAttrModified(aEvent) {
-    if (Tabmix.isVersion(40)) {
-      let tab = aEvent.target;
-      this._setMenuitemAttributes(tab.mCorrespondingMenuitem, tab);
-      return;
-    }
-
-    var menuItem = aEvent.target.mCorrespondingMenuitem;
-    if (menuItem) {
-      var attrName = aEvent.attrName;
-      var count = "";
-      switch (attrName) {
-        case "label":
-          count = menuItem.getAttribute("count");
-        case "crop":
-        case "busy":
-        case "image":
-        case "selected":
-          if (aEvent.attrChange == aEvent.REMOVAL)
-            menuItem.removeAttribute(attrName);
-          else
-            menuItem.setAttribute(attrName, count + aEvent.newValue);
-      }
-    }
-  },
-
   _setMenuitemAttributes: function TMP__setMenuitemAttributes(aMenuitem, aTab) {
     aMenuitem.setAttribute("label", aMenuitem.getAttribute("count") + aTab.label);
     aMenuitem.setAttribute("crop", aTab.getAttribute("crop"));
@@ -1072,9 +1006,9 @@ var TabmixAllTabs = {
 
   _tabSelectedFromList: function TMP__tabSelectedFromList(aTab) {
     if (gBrowser.selectedTab == aTab)
-      gBrowser.tabContainer.ensureTabIsVisible(gBrowser.tabContainer.selectedIndex);
+      gBrowser.tabContainer.mTabstrip.ensureElementIsVisible(aTab);
     else
-      // if we select another tab _handleTabSelect will call ensureTabIsVisible
+      // if we select another tab _handleTabSelect will call mTabstrip.ensureElementIsVisible
       gBrowser.selectedTab = aTab;
   },
 
@@ -1085,8 +1019,6 @@ var TabmixAllTabs = {
       if (menuItem.id.indexOf("btn_tabslist") != -1)
         break;
       if ("tab" in menuItem) {
-        if (!Tabmix.isVersion(40))
-          menuItem.tab.removeEventListener("DOMAttrModified", this, false);
         menuItem.tab.mCorrespondingMenuitem = null;
       }
       popup.removeChild(menuItem);
@@ -1096,22 +1028,19 @@ var TabmixAllTabs = {
     if (item.id == "btn_tabslist" || item.id == "btn_undoclose")
       item.setAttribute('tooltiptext', item.getAttribute('_tooltiptext'));
 
-    if (Tabmix.isVersion(40))
-      gBrowser.tabContainer.removeEventListener("TabAttrModified", this, false);
+    gBrowser.tabContainer.removeEventListener("TabAttrModified", this, false);
+    gBrowser.tabContainer.mTabstrip.removeEventListener("scroll", this, false);
     gBrowser.tabContainer.removeEventListener("TabClose", this, false);
     popup.removeEventListener("DOMMenuItemActive", this, false);
     popup.removeEventListener("DOMMenuItemInactive", this, false);
-    popup.removeEventListener("DOMMouseScroll", this, true);
 
     this.backupLabel = "";
     this._selectedItem = null;
+    popup._updateTabsVisibilityStatus = null;
+    this._popup = null;
   },
 
   updateMenuItemActive: function TMP_updateMenuItemActive(event, tab) {
-    if (!Tabmix.isVersion(40)) {
-      let statusTextFld = document.getElementById("statusbar-display");
-      if (this.backupLabel=="") this.backupLabel = statusTextFld.label;
-    }
     if (!tab)
       tab = event.target;
     this.updateStatusText(tab.getAttribute("statustext"));
@@ -1122,13 +1051,6 @@ var TabmixAllTabs = {
   },
 
   updateStatusText: function TMP_updateStatusText(itemText) {
-    if (Tabmix.isVersion(40))
-      XULBrowserWindow.setOverLink(itemText);
-    else {
-      var statusTextFld = document.getElementById("statusbar-display");
-      var newText = itemText ? itemText : this.backupLabel;
-      if (newText != statusTextFld.label)
-        statusTextFld.label = newText;
-    }
+    XULBrowserWindow.setOverLink(itemText);
   }
 }
