@@ -11,12 +11,12 @@
 var TMP_extensionsCompatibility = {
   preInit: function TMP_EC_preInit() {
     if ("TreeStyleTabWindowHelper" in window) {
-      TMP_TreeStyleTab.preInit();
+      this.treeStyleTab.preInit();
     }
   },
 
   onContentLoaded: function TMP_EC_onContentLoaded() {
-    Tabmix.extensions = {sessionManager: false, treeStyleTab: false, tabGroupManager: false};
+    Tabmix.extensions = {sessionManager: false, treeStyleTab: false, tabGroupManager: false, verticalTabBar: false};
     try {
       if ("TabGroupsManagerApiVer1" in window) {
         Tabmix.extensions.tabGroupManager = true;
@@ -25,7 +25,7 @@ var TMP_extensionsCompatibility = {
         let tmp = {};
         Components.utils.import("resource://tabmixplus/extensions/TabGroupsManager.jsm", tmp);
         tmp.TMP_TabGroupsManager.newCode = Tabmix.newCode;
-        tmp.TMP_TabGroupsManager.init(window, gBrowser.tabContainer, Tabmix.isVersion(40));
+        tmp.TMP_TabGroupsManager.init(window, gBrowser.tabContainer);
       }
     } catch (ex) {Tabmix.assert(ex, "error in TabGroupsManager.jsm");}
 
@@ -40,7 +40,7 @@ var TMP_extensionsCompatibility = {
           Tabmix.newCode("GlaxChrome.CLT.DragDropManager." + aFn , GlaxChrome[aFn])._replace(
             '{', '{var TabDNDObserver = TMP_tabDNDObserver;', {check: GlaxChrome[aFn].toString().indexOf("TabDNDObserver") != -1}
           )._replace(
-            'TM_init', 'TMP_startup', {check: GlaxChrome[aFn].toString().indexOf("TM_init") != -1, flags: "g"}
+            'TM_init', 'Tabmix.startup', {check: GlaxChrome[aFn].toString().indexOf("TM_init") != -1, flags: "g"}
           ).toCode();
         }
       });
@@ -58,9 +58,12 @@ var TMP_extensionsCompatibility = {
     }
 
     try {
-      if ("TreeStyleTabService" in window)
-        TMP_TreeStyleTab.onContentLoaded();
-    } catch (ex) {Tabmix.assert(ex, TMP_TreeStyleTab.errorMsg);}
+      if ("TreeStyleTabService" in window) {
+        this.treeStyleTab.onContentLoaded();
+        Tabmix.extensions.treeStyleTab = true;
+        Tabmix.extensions.verticalTabBar = true;
+      }
+    } catch (ex) {Tabmix.assert(ex, this.treeStyleTab.errorMsg);}
 
     // https://addons.mozilla.org/en-US/firefox/addon/second-search/
     if ("SecondSearchBrowser" in window && SecondSearchBrowser.prototype) {
@@ -69,7 +72,7 @@ var TMP_extensionsCompatibility = {
       func.forEach(function(aFn) {
         if (aFn in SSB && SSB[aFn].toString().indexOf("TM_init") != -1) {
           Tabmix.newCode("SecondSearchBrowser.prototype." + aFn , SSB[aFn])._replace(
-            'TM_init', 'TMP_startup'
+            'TM_init', 'Tabmix.startup'
           ).toCode();
         }
       });
@@ -80,7 +83,7 @@ var TMP_extensionsCompatibility = {
       let _getter = SplitBrowser.__lookupGetter__("tabbedBrowsingEnabled");
       if (_getter.toString().indexOf("TM_init") != -1) {
         Tabmix.newCode(null,  _getter)._replace(
-          'TM_init', 'TMP_startup'
+          'TM_init', 'Tabmix.startup'
         ).toGetter(SplitBrowser, "tabbedBrowsingEnabled");
       }
     }
@@ -92,38 +95,101 @@ var TMP_extensionsCompatibility = {
             getBoolPref.toString().indexOf("return bgSaverPref.prefHasUserValue(sName)") != -1) {
       window.getBoolPref = function getBoolPref ( prefname, def ) {
         try {
-          return TabmixSvc.prefs.getBoolPref(prefname);
+          return Services.prefs.getBoolPref(prefname);
         }
         catch(er) { return def; }
       }
     }
 
-    // https://addons.mozilla.org/en-US/firefox/addon/bug489729-disable-detach-and-t//
-    if ("bug489729" in window) {
-      Tabmix.newCode("bug489729.init", bug489729.init)._replace(
-        '"TabDNDObserver" in window', 'true', {silent: true}
-      )._replace(
-        'TabDNDObserver', 'TMP_tabDNDObserver', {silent: true, flags: "g"}
-      )._replace(
-        'if (gIsFirefox35) {',
-        'this._dragLeftWindow = false;', {silent: true}
-      )._replace(
-        'gSingleWindowMode', 'Tabmix.singleWindowMode', {silent: true}
+    /*  we don't use this code - leave it here as a reminder.
+
+    // workaround for extensions that look for updateIcon
+    // Favicon Picker 2
+    if (typeof(gBrowser.updateIcon) == "undefined") {
+      gBrowser.updateIcon = function updateIcon (aTab) {
+        var browser = gBrowser.getBrowserForTab(aTab);
+        if ((browser.mIconURL || "") != aTab.getAttribute("image")) {
+          if (browser.mIconURL)
+            aTab.setAttribute("image", browser.mIconURL);
+          else
+            aTab.removeAttribute("image");
+          gBrowser._tabAttrModified(aTab);
+        }
+      }
+    }
+    */
+
+    /*
+    // https://addons.mozilla.org/en-US/firefox/addon/tab-flick/
+    if ("TabFlick" in window && typeof(TabFlick.openPanel) == "function") {
+      Tabmix.newCode("TMP_tabDNDObserver.onDragEnd", TMP_tabDNDObserver.onDragEnd)._replace(
+        'gBrowser.replaceTabWithWindow(draggedTab);',
+        'gBrowser.selectedTab = draggedTab; TabFlick.openPanel(aEvent);'
       ).toCode();
     }
+    */
 
+
+    // https://addons.mozilla.org/en-US/firefox/addon/bug489729-disable-detach-and-t//
+    // we don't need to do any changes to bug489729 extension version 1.6+
+
+
+    // https://addons.mozilla.org/en-US/firefox/addon/foxtab/
+    if ("foxTab" in window) {
+      let loadNewInBackground = '$& var loadNewInBackground = Tabmix.prefs.getBoolPref("loadNewInBackground");';
+      let newCode = <![CDATA[
+        if (Tabmix.prefs.getBoolPref("openNewTabNext"))
+          f.gBrowser.moveTabTo(newTab, f.gBrowser.selectedTab._tPos + 1);
+        if (!loadNewInBackground) {
+          f.gBrowser.TMP_selectNewForegroundTab(newTab, false);
+          TMP_LastTab.PushSelectedTab();
+        }
+      ]]>
+      if (typeof(foxTab.openNewTab) == "function") {
+        Tabmix.newCode("foxTab.openNewTab", foxTab.openNewTab)._replace(
+          '{', loadNewInBackground
+        )._replace(
+          'f.isFlashOpen',
+          'f.isFlashOpen && !loadNewInBackground'
+        )._replace(
+          'f.disableTabSelectedNewTab = true;',
+          'f.disableTabSelectedNewTab = !loadNewInBackground;'
+        )._replace(
+          'f.gBrowser.selectedTab = newTab', newCode
+        )._replace(
+          'f.addBrowserListener(f.gBrowser.selectedBrowser);',
+          'if( !loadNewInBackground) $&'
+        ).toCode();
+      }
+      if (typeof(foxTab.showNewTabMessage) == "function") {
+        Tabmix.newCode("foxTab.showNewTabMessage", foxTab.showNewTabMessage)._replace(
+          '{', loadNewInBackground
+        )._replace(
+          'f.gBrowser.selectedTab = newTab', newCode
+        ).toCode();
+      }
+      window.BrowserOpenTab = TMP_BrowserOpenTab;
+      foxTab.defaultBrowserOpenTab = TMP_BrowserOpenTab;
+    }
   },
 
   onWindowOpen: function TMP_EC_onWindowOpen() {
+    // https://addons.mozilla.org/EN-US/firefox/addon/vertical-tabs/
+    // https://addons.mozilla.org/EN-US/firefox/addon/side-tabs/
+    if (typeof VerticalTabs == "object" || typeof sidetabs == "object") {
+      Tabmix.extensions.verticalTabs = true;
+      Tabmix.extensions.verticalTabBar = true;
+    }
+
     // Look for RSS/Atom News Reader
     if ("gotoLink" in window)
-      TMP_wizzrss.init();
+      this.wizzrss.init();
 
     if ("openNewsfox" in window)
-      TMP_Newsfox.init();
+      this.newsfox.init();
 
     if ("RSSTICKER" in window)
-      TMP_RSSTICKER.init();
+      this.RSSTICKER.init();
 
     if ("PersonaController" in window && typeof(window.PersonaController) == "object") {
       Tabmix.newCode('PersonaController._applyPersona', PersonaController._applyPersona)._replace(
@@ -162,7 +228,7 @@ var TMP_extensionsCompatibility = {
         'if (bOpenWindow && !Tabmix.singleWindowMode)'
       )._replace(
         'if (bOpenTab)',
-        'if (bOpenTab || TMP_whereToOpen(null).lock)'
+        'if (bOpenTab || Tabmix.whereToOpen(null).lock)'
       ).toCode();
     }
 
@@ -206,15 +272,17 @@ var TMP_extensionsCompatibility = {
     } catch (ex) {}
 
     // make URL Suffix extension compatible with tabmix
-    if ("objURLsuffix" in window) {
+    if ("objURLsuffix" in window && !Tabmix.isVersion(100)) {
       if ("handleURLBarCommand" in objURLsuffix) {
         Tabmix.newCode("objURLsuffix.handleURLBarCommand", objURLsuffix.handleURLBarCommand)._replace(
           'objURLsuffix.BrowserLoadURL(aTriggeringEvent, postData.value, altDisabled);',
-          'TMP_BrowserLoadURL(aTriggeringEvent, postData.value, altDisabled);'
+          'Tabmix.browserLoadURL(aTriggeringEvent, postData.value, altDisabled);'
         )._replace(
           'objURLsuffix.BrowserLoadURL(aTriggeringEvent, postData.value);',
-          'TMP_BrowserLoadURL(aTriggeringEvent, postData.value, true);'
+          'Tabmix.browserLoadURL(aTriggeringEvent, postData.value, true);'
         ).toCode();
+
+        window.handleURLBarCommand = objURLsuffix.handleURLBarCommand;
       }
 
       if ("canonizeUrl" in objURLsuffix) {
@@ -223,20 +291,18 @@ var TMP_extensionsCompatibility = {
           'return [gURLBar.value, aPostDataRef, true];'
         ).toCode();
       }
-
-      window.handleURLBarCommand = objURLsuffix.handleURLBarCommand;
     }
 
     try {
       if ("TreeStyleTabService" in window)
-        TMP_TreeStyleTab.onWindowLoaded();
-    } catch (ex) {Tabmix.assert(ex, TMP_TreeStyleTab.errorMsg);}
+        this.treeStyleTab.onWindowLoaded();
+    } catch (ex) {Tabmix.assert(ex, this.treeStyleTab.errorMsg);}
 
     /* fast dial FdUtils*/
     if ("FdUtils" in window && FdUtils.whereToOpenLink) {
       Tabmix.newCode("FdUtils.whereToOpenLink", FdUtils.whereToOpenLink)._replace(
         'if (e.ctrlKey)',
-        'if (e.ctrlKey || TMP_whereToOpen(null).lock)'
+        'if (e.ctrlKey || Tabmix.whereToOpen(null).lock)'
       ).toCode();
     }
 
@@ -247,7 +313,7 @@ var TMP_extensionsCompatibility = {
       Tabmix.newCode("Local_Install.openThrobber", Local_Install.openThrobber)._replace(
         'local_common.openURL(local_common.getThrobberURL(), inNewTab);',
         'var url = local_common.getThrobberURL(); \
-         local_common.openURL(url, inNewTab ? inNewTab : TMP_checkCurrent(url) == "tab");'
+         local_common.openURL(url, inNewTab ? inNewTab : Tabmix.checkCurrent(url) == "tab");'
       ).toCode();
       // add name to closeallOverlay.onUnload we use it in goQuitApplication
       if ("closeallOverlay" in window && "onUnload" in closeallOverlay) {
@@ -293,44 +359,52 @@ var TMP_extensionsCompatibility = {
       ).toCode();
     }
 
+    // Redirect Remover 2.6.4
+    if ("rdrb" in window && rdrb.delayedInit) {
+      Tabmix.newCode("TabmixContext.openMultipleLinks", TabmixContext.openMultipleLinks)._replace(
+        'if (links_urlSecurityCheck(url))',
+        'url = rdrb.cleanLink(url);\
+         $&'
+      ).toCode();
+    }
+
+    // override some of All-in-One Gestures function
+    // override the duplicate tab function
+    if (typeof aioDupTab == 'function')
+      aioDupTab = function() { gBrowser.duplicateTab(gBrowser.mCurrentTab); };
+
+    // override the duplicate in new window function
+    if (typeof aioDupWindow == 'function')
+      aioDupWindow = function() { gBrowser.duplicateInWindow(gBrowser.mCurrentTab); };
+
+    // override the aioCloseWindow function
+    if (typeof aioCloseWindow == 'function')
+      aioCloseWindow = BrowserTryToCloseWindow;
+
   },
 
   onDelayedStartup: function TMP_EC_onDelayedStartup() {
-    // some themes uses old tabmix_3.xml version and call _createTabsList
-    function _fixOnContextmenu(item, side) {
-      if (side == "right")
-        item.oncontextmenu = function (event) {TabmixAllTabs.createScrollButtonTabsList(event, "right");}
-      else
-        item.oncontextmenu = function (event) {TabmixAllTabs.createScrollButtonTabsList(event, "left");}
-      item.removeAttribute("oncontextmenu");
-    }
-
-    var buttonDown = document.getAnonymousElementByAttribute(gBrowser.tabContainer, "anonid", "scrollbutton-down") ||
-                     document.getAnonymousElementByAttribute(gBrowser.tabContainer.mTabstrip, "anonid", "scrollbutton-down");
-    if (buttonDown && buttonDown.oncontextmenu){
-      _fixOnContextmenu(buttonDown, "right");
-      _fixOnContextmenu(buttonDown.previousSibling, "left");
-      let buttonUp = document.getAnonymousElementByAttribute(gBrowser.tabContainer, "anonid", "scrollbutton-up") ||
-                     document.getAnonymousElementByAttribute(gBrowser.tabContainer.mTabstrip, "anonid", "scrollbutton-up");
-      if (buttonUp && buttonUp.oncontextmenu)
-        _fixOnContextmenu(buttonUp, "left");
-    }
+    //XXX some themes uses old tabmix_3.xml version and call _createTabsList
+    // for now we don't do changes for other themes regarding scroll buttons oncontextmenu
+    // with the new code in scrollbox.xml
+    // this need more testing with other themes
 
     // check if Greasemonkey installed
-    TMP_isGreasemonkeyInstalled();
+    Tabmix.contentAreaClick.isGreasemonkeyInstalled();
   }
+
 }
 
-var TMP_RSSTICKER = {
+TMP_extensionsCompatibility.RSSTICKER = {
    init : function ()  {
-     Tabmix.newCode("RSSTICKER.writeFeed ", RSSTICKER.writeFeed)._replace(
+     Tabmix.newCode("RSSTICKER.writeFeed", RSSTICKER.writeFeed)._replace(
        'tbb.setAttribute("onclick"',
        'tbb.setAttribute("onclick", "this.onClick(event);");\
         tbb.setAttribute("_onclick"'
      )._replace(
        'tbb.onContextOpen =',
-       'tbb.onContextOpen = TMP_RSSTICKER.onContextOpen; \
-        tbb.onClick = TMP_RSSTICKER.onClick; \
+       'tbb.onContextOpen = TMP_extensionsCompatibility.RSSTICKER.onContextOpen; \
+        tbb.onClick = TMP_extensionsCompatibility.RSSTICKER.onClick; \
         tbb._onContextOpen ='
      ).toCode();
    },
@@ -349,7 +423,7 @@ var TMP_RSSTICKER = {
 
    onContextOpen : function (target) {
      if (!target) {
-       if (TMP_whereToOpen(null).lock)
+       if (Tabmix.whereToOpen(null).lock)
          this.parent.browser.openInNewTab(this.href);
        else
          window._content.document.location.href = this.href;
@@ -369,61 +443,56 @@ var TMP_RSSTICKER = {
 }
 
 // prevent Wizz RSS from load pages in locked tabs
-var TMP_wizzrss = {
-   started: null,
-   init : function ()  {
-      if (this.started)
-        return;
-      this.started = true;
-      var codeToReplace = /getContentBrowser\(\).loadURI|contentBrowser.loadURI/g;
-      const newCode = "TMP_wizzrss.openURI";
-      var _functions = ["addFeedbase","validate","gohome","tryagain","promptStuff",
-                        "doSearch","viewLog","renderItem","playEnc","renderAllEnc","playAllEnc",
-                        "gotoLink","itemLinkClick","itemListClick"];
+TMP_extensionsCompatibility.wizzrss = {
+  started: null,
+  init : function ()  {
+    if (this.started)
+      return;
+    this.started = true;
+    var codeToReplace = /getContentBrowser\(\).loadURI|contentBrowser.loadURI/g;
+    const newCode = "TMP_extensionsCompatibility.wizzrss.openURI";
+    var _functions = ["addFeedbase","validate","gohome","tryagain","promptStuff",
+                      "doSearch","viewLog","renderItem","playEnc","renderAllEnc","playAllEnc",
+                      "gotoLink","itemLinkClick","itemListClick"];
 
-      _functions.forEach(
-         function(_function) {
-            if (_function in window)
-               Tabmix.newCode("window." + _function, window[_function])._replace(codeToReplace,newCode).toCode();
-         }
-      );
-   },
+    _functions.forEach(function(_function) {
+      if (_function in window)
+        Tabmix.newCode("window." + _function, window[_function])._replace(codeToReplace,newCode).toCode();
+    });
+  },
 
-   openURI : function (uri)  {
-      var w = Tabmix.getTopWin();
-      var tabBrowser = w.gBrowser;
+  openURI : function (uri)  {
+    var w = Tabmix.getTopWin();
+    var tabBrowser = w.gBrowser;
 
-      var openNewTab = w.TMP_whereToOpen(true).lock;
-      if (openNewTab) {
-         var theBGPref = !readPref("WizzRSSFocusTab", false, 2);
-         if (Tabmix.isVersion(36))
-           tabBrowser.loadOneTab(uri, {inBackground: theBGPref});
-         else
-           tabBrowser.loadOneTab(uri, null, null, null, theBGPref);
-      }
-      else
-         tabBrowser.loadURI(uri);
-   }
+    var openNewTab = w.Tabmix.whereToOpen(true).lock;
+    if (openNewTab) {
+      var theBGPref = !readPref("WizzRSSFocusTab", false, 2);
+      tabBrowser.loadOneTab(uri, {inBackground: theBGPref});
+    }
+    else
+      tabBrowser.loadURI(uri);
+  }
 }
 
 // prevent Newsfox from load pages in locked tabs
-var TMP_Newsfox = {
+TMP_extensionsCompatibility.newsfox = {
    init : function ()  {
-      Tabmix.newCode("openNewsfox ", openNewsfox)._replace(
+      Tabmix.newCode("openNewsfox", openNewsfox)._replace(
          /if \(newTab\) {/,
-         'newTab = newTab || TMP_whereToOpen(null).lock; \
+         'newTab = newTab || Tabmix.whereToOpen(null).lock; \
          $&'
       ).toCode();
    }
 }
 
 /**
- * fix incompatibilities with TMP_TreeStyleTab
+ * fix incompatibilities with treeStyleTab
  * we get here only if "TreeStyleTabService" exist in window
  *
  *  https://addons.mozilla.org/en-US/firefox/addon/tree-style-tab/
  */
-var TMP_TreeStyleTab = {
+TMP_extensionsCompatibility.treeStyleTab = {
   errorMsg: "Error in Tabmix when trying to load compatible functions with TreeStyleTab extension",
   // flag for version after 2011.01.13.01
   isNewVersion: function () {
@@ -437,9 +506,9 @@ var TMP_TreeStyleTab = {
     if (typeof TreeStyleTabWindowHelper.overrideExtensionsPreInit == "function") {
       Tabmix.newCode("TreeStyleTabWindowHelper.overrideExtensionsPreInit",
           TreeStyleTabWindowHelper.overrideExtensionsPreInit)._replace(
-        // fix typo in TST
+        // fix typo in TST - this typo fixed on TST 0.13.2011121501
         'SessionData.tabTSTProperties',
-        'sessionData.tabTSTProperties'
+        'sessionData.tabTSTProperties', {silent: true}
       )._replace(
         // TST look for DEPRECATED gBrowser.restoreTab in tablib.init
         'source[1].replace',
@@ -456,7 +525,6 @@ var TMP_TreeStyleTab = {
       if ("overrideExtensionsOnInitAfter" in TreeStyleTabService) {
         // TMupdateSettings replaced with TabmixTabbar.updateSettings
         // TabDNDObserver replaced with TMP_tabDNDObserver
-        // TM_BrowserHome replaced with TMP_BrowserHome
         // tabBarScrollStatus replaced with TabmixTabbar.updateScrollStatus
 
         Tabmix.newCode("TreeStyleTabService.overrideExtensionsOnInitAfter", TreeStyleTabService.overrideExtensionsOnInitAfter)._replace(
@@ -464,7 +532,7 @@ var TMP_TreeStyleTab = {
         )._replace(
           'this.updateTabDNDObserver(TabDNDObserver);',
           'this.updateTabDNDObserver(gBrowser);'
-        )._replace( /* we don't need it, tabmix TMP_BrowserHome calls gBrowser.loadTabs */
+        )._replace( /* we don't need it, tabmix Tabmix.browserHome calls gBrowser.loadTabs */
           'eval("window.TM_BrowserHome = "',
           'if (false) $&'
         )._replace(
@@ -483,15 +551,6 @@ var TMP_TreeStyleTab = {
           'TabmixTabbar.getRowHeight', {flags: "g"}
         ).toCode();
 
-        // we add tabbrowser to tabContainer before Firefox 4.0
-        Tabmix.newCode("TreeStyleTabService.updateTabDNDObserver", TreeStyleTabService.updateTabDNDObserver)._replace(
-          'aObserver.tabContainer.tabbrowser == aObserver',
-          'Tabmix.isVersion(40) && $&', {check: !Tabmix.isVersion(40)}
-        )._replace(
-          'let locked =',
-          'var locked =', {flags: "g", silent: true}
-        ).toCode();
-
         /* for treeStyleTab extension look in treeStyleTab hacks.js
            we remove tabxTabAdded function and use TMP_eventListener.onTabOpen from 0.3.7pre.080815
         */
@@ -502,16 +561,6 @@ var TMP_TreeStyleTab = {
         }
       }
 
-      // TMupdateSettings replaced with TabmixTabbar.updateSettings
-      if ("overrideExtensionsDelayed" in TreeStyleTabService) {
-        Tabmix.newCode("TreeStyleTabService.overrideExtensionsDelayed", TreeStyleTabService.overrideExtensionsDelayed)._replace(
-          '"TMupdateSettings" in window', 'true', {silent: true} // not in use from TST 2011.01.13.01
-        )._replace(
-          'TM_BrowserHome',
-          'TMP_BrowserHome', {silent: true} // not in use from TST 2011.01.13.01
-        ).toCode();
-      }
-
       if ('piro.sakura.ne.jp' in window && "tabsDragUtils" in window['piro.sakura.ne.jp']) {
         let tabsDragUtils = window['piro.sakura.ne.jp'].tabsDragUtils;
         Tabmix.newCode("tabsDragUtils._delayedInit", tabsDragUtils._delayedInit)._replace(
@@ -539,9 +588,17 @@ var TMP_TreeStyleTab = {
       }
     }
 
+    // we removed TMP_howToOpen function 2011-11-15
+    if ("TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit) {
+      Tabmix.newCode("TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit",
+          TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit)._replace(
+        'eval("window.TMP_howToOpen',
+        'if (false) $&'
+      ).toCode();
+    }
+
     // we removed TMP_openTabNext function 2011-11-15
-    if (!("TMP_openTabNext" in gBrowser) &&
-        "TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsDelayed) {
+    if ("TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsDelayed) {
       Tabmix.newCode("TreeStyleTabWindowHelper.overrideExtensionsDelayed",
           TreeStyleTabWindowHelper.overrideExtensionsDelayed)._replace(
         'eval("gBrowser.TMP_openTabNext',
@@ -551,33 +608,11 @@ var TMP_TreeStyleTab = {
   },
 
   onWindowLoaded: function () {
-    function replaceHorizontalProps(aObjectName, aObject) {
-      var ensureTabIsVisible = aObjectName == "gBrowser.tabContainer.ensureTabIsVisible";
-      Tabmix.newCode(aObjectName, aObject)._replace(
-        'scrollBoxObject.width < 250',
-        '$& || gBrowser.treeStyleTab.isVertical', {check: ensureTabIsVisible && this.isNewVersion, flags: "g"}
-      )._replace(
-        'scrollBoxObject.screenX',
-        'scrollBoxObject[TMP_screenPosProp]', {flags: "g"}
-      )._replace(
-        'scrollBoxObject.width',
-        'scrollBoxObject[sizeProp]', {check: this.isNewVersion, flags: "g"}
-      )._replace(
-        '{',
-        '{var TMP_screenPosProp = gBrowser.treeStyleTab.isVertical ? "screenY" : "screenX";\
-          var TMP_sizeProp = gBrowser.treeStyleTab.isVertical ? "height" : "width";'
-      ).toCode();
-    }
-    if (!TreeStyleTabService.getTreePref("TMP.doNotUpdate.isTabVisible"))
-      replaceHorizontalProps("gBrowser.tabContainer.isTabVisible", gBrowser.tabContainer.isTabVisible);
-
-    replaceHorizontalProps("gBrowser.tabContainer.ensureTabIsVisible", gBrowser.tabContainer.ensureTabIsVisible);
-
     /**
      *  TST have eval to TMP_Bookmark.openGroup
      *  we replace TMP_Bookmark.openGroup with TMP_Places.openGroup at Tabmix 0.3.8.2pre.090830
      *  we also replace call to TreeStyleTabService.openGroupBookmarkBehavior();
-     *  with aOpenGroupBookmarkBehavior that we pass from  PlacesUIUtils._openTabset
+     *  with aOpenGroupBookmarkBehavior that we pass from PlacesUIUtils._openTabset
      *  we only call this functiom from browserWindow so we don't need to call it for
      *  other places windows
      */
@@ -603,15 +638,15 @@ var TMP_TreeStyleTab = {
         }
       ]]>
     )._replace(
-      'tabBar.nextTab',
+      /(\})(\)?)$/,
       <![CDATA[
         if (TSTOpenGroupBookmarkBehavior & TreeStyleTabService.kGROUP_BOOKMARK_SUBTREE)
           TreeStyleTabService.stopToOpenChildTab(tabToSelect);
-      $&]]>
+      $1$2]]>
     ).toCode();
 
-    // Added 2010-04-10
     if (TreeStyleTabService.getTreePref('compatibility.TMP')) {
+      // Added 2010-04-10
       // TST look for aTab.removeAttribute("tabxleft")
       Tabmix.newCode("window.TabmixTabbar.updateSettings", TabmixTabbar.updateSettings)._replace(
         'TabmixSessionManager.updateTabProp(aTab);',
@@ -619,13 +654,18 @@ var TMP_TreeStyleTab = {
          gBrowser.treeStyleTab.initTabAttributes(aTab);\
          gBrowser.treeStyleTab.initTabContentsOrder(aTab);'
       ).toCode();
-
+      // Added 2010-04-10
       Tabmix.newCode("TMP_eventListener.onTabOpen", TMP_eventListener.onTabOpen)._replace(
         /(\})(\)?)$/,
         'gBrowser.treeStyleTab.initTabAttributes(tab); \
          gBrowser.treeStyleTab.initTabContentsOrder(tab); \
          $1$2'
       ).toCode();
+      // Added 2011-11-09, i'm not sure we really need it, Tabmix.loadTabs call gBrowser.loadTabs
+      Tabmix.newCode("TabmixContext.openMultipleLinks", TabmixContext.openMultipleLinks)._replace(
+        /(Tabmix.loadTabs\([^\)]+\);)/g,
+        'TreeStyleTabService.readyToOpenChildTab(gBrowser, true); $1 TreeStyleTabService.stopToOpenChildTab(gBrowser);'
+      ).toCode();
     }
   }
 }
