@@ -9,39 +9,8 @@
  *
  */
 
-/**
- * @brief Catch call to tabmix options from EM
- *        we only use Tabmix options if we have browser window
- *
- * @param aURL       A valid options URL string.
- *
- * @return           true if the URL is for tabmix options
- *                   false if not.
- *
- */
-function TMP_cmd_options(aURL) {
-   if (aURL != "chrome://tabmixplus/content/pref/pref-tabmix.xul")
-      return false;
-
-   var windowMediator = TabmixSvc.wm;
-   var browserWindow = windowMediator.getMostRecentWindow('navigator:browser');
-
-   if (!browserWindow) {
-      var tabmixopt = windowMediator.getMostRecentWindow("mozilla:tabmixopt");
-      if (tabmixopt)
-         tabmixopt.close();
-      var title = TabmixSvc.getString("tabmixoption.error.title");
-      var msg = TabmixSvc.getString("tabmixoption.error.msg");
-      TabmixSvc.prompt.alert(window, title, msg);
-   }
-   else
-      browserWindow.TMP_openDialog(-1);
-
-   return true;
-}
-
-function TMP_openDialog(panel) {
-  var windowMediator = TabmixSvc.wm;
+Tabmix.openOptionsDialog = function TMP_openDialog(panel) {
+  var windowMediator = Services.wm;
   var tabmixOptionsWin = windowMediator.getMostRecentWindow("mozilla:tabmixopt");
   if (tabmixOptionsWin) {
     var appearanceWin = windowMediator.getMostRecentWindow("mozilla:tabmixopt-appearance");
@@ -57,7 +26,7 @@ function TMP_openDialog(panel) {
   }
   else {
     if(panel > -1)
-      TabmixSvc.prefs.setIntPref('extensions.tabmix.selected_tab', panel);
+      Tabmix.prefs.setIntPref("selected_tab", panel);
 
     window.openDialog("chrome://tabmixplus/content/pref/pref-tabmix.xul", "Tab Mix Plus", "chrome,titlebar,toolbar,close,dialog=no");
   }
@@ -74,10 +43,10 @@ function TMP_openDialog(panel) {
  * @return           true.
  *
  */
-function TMP_openURL(aURL, event) {
+Tabmix.openURL = function TMP_openURL(aURL, event) {
    var linkTarget, loadInBackground;
    try {
-            linkTarget = TabmixSvc.prefs.getIntPref("browser.link.open_newwindow");
+            linkTarget = Services.prefs.getIntPref("browser.link.open_newwindow");
    }
    catch (e) {
       linkTarget = 1;
@@ -86,7 +55,7 @@ function TMP_openURL(aURL, event) {
    if (aURL == null) aURL = "about:blank";
 
    // check for an existing window and focus it; it's not application modal
-   var browserWindow = Tabmix.getTopWin();
+   var browserWindow = this.getTopWin();
 
    if (!browserWindow) {
       openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", aURL, null, null, null);
@@ -116,7 +85,7 @@ function TMP_openURL(aURL, event) {
             break;
          case 3 :
             // added by request, for extensions with multiple homepages
-            browserWindow.TMP_loadTabs(aURL.split("|"), false);
+            browserWindow.Tabmix.loadTabs(aURL.split("|"), false);
             break;
       }
 
@@ -127,10 +96,13 @@ function TMP_openURL(aURL, event) {
    return true;
 }
 
+// Don't change this function name other extensions using it
+// Speed-Dial, Fast-Dial, TabGroupManager
 function TMP_BrowserOpenTab(aTab, replaceLastTab) {
-   var newTabContent = replaceLastTab ? TabmixSvc.TMPprefs.getIntPref("replaceLastTabWith") :
-                                         TabmixSvc.TMPprefs.getIntPref("loadOnNewTab");
+   var newTabContent = replaceLastTab ? Tabmix.prefs.getIntPref("replaceLastTabWith.type") :
+                                        Tabmix.prefs.getIntPref("loadOnNewTab.type");
    var url;
+   var newTabUrl = Tabmix.newTabURL;
    switch (newTabContent) {
       case 0 : // blank tab, by default
          url = "about:blank";
@@ -140,31 +112,38 @@ function TMP_BrowserOpenTab(aTab, replaceLastTab) {
          break;
       case 2 : // current URI
          var currentURI = gBrowser.currentURI;
-         url = currentURI ? currentURI.spec : "about:blank";
+         url = currentURI ? currentURI.spec : newTabUrl;
          break;
       case 3 : // duplicate tab
          let currentUrl = gBrowser.currentURI.spec;
          let newTab = gBrowser.duplicateTab(gBrowser.mCurrentTab, null, null, null, true);
-         TMP_clearUrlBar(newTab, currentUrl, true);
+         Tabmix.clearUrlBar(newTab, currentUrl, true);
          return newTab;
          break;
       case 4 : // user url
+         let prefName;
+         if (replaceLastTab) {
+           prefName = typeof isBlankPageURL == "function" ?
+                "extensions.tabmix.replaceLastTabWith.newtab.url" :
+                "extensions.tabmix.replaceLastTabWith.newTabUrl"
+         }
+         else
+           prefName = Tabmix.newTabURLpref;
          try {
-            var prefName = replaceLastTab ? "newTabUrl_afterLastTab" : "newTabUrl";
-            url = TabmixSvc.TMPprefs.getComplexValue(prefName, Components.interfaces.nsIPrefLocalizedString).data;
+            url = Services.prefs.getComplexValue(prefName, Components.interfaces.nsISupportsString).data;
          } catch (ex) {  Tabmix.assert(ex); }
          // use this if we can't find the pref
          if (!url)
-            url = "about:blank";
+            url = newTabUrl;
          break;
       default:
-         url = "about:blank";
+         url = newTabUrl;
    }
    var flags = nsIWebNavigation.LOAD_FLAGS_NONE;
    // if google.toolbar extension installed check google.toolbar.newtab pref
    if ("GTB_GoogleToolbarOverlay" in window) {
      try {
-       if (TabmixSvc.prefs.getBoolPref("google.toolbar.newtab")) {
+       if (Services.prefs.getBoolPref("google.toolbar.newtab")) {
          url = "chrome://google-toolbar/content/new-tab.html";
          flags = nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY
        }
@@ -173,11 +152,12 @@ function TMP_BrowserOpenTab(aTab, replaceLastTab) {
    if ((TabmixTabbar.widthFitTitle || !Tabmix.isVersion(60)) && replaceLastTab && !gBrowser.mCurrentTab.collapsed)
      gBrowser.mCurrentTab.collapsed = true;
 
-   var loadBlank = url == "about:blank";
-   var newTab = replaceLastTab && Tabmix.isVersion(40) ? gBrowser.addTab("about:blank", {skipAnimation: true}) : gBrowser.addTab("about:blank");
+   var loadBlank = Tabmix.isBlankPageURL(url);
+   var _url = loadBlank ? url : "about:blank";
+   var newTab = replaceLastTab ? gBrowser.addTab(_url, {skipAnimation: true}) : gBrowser.addTab(_url);
    if (replaceLastTab) {
      newTab.__newLastTab = true;
-     if (TabmixSvc.prefs.getCharPref("general.skins.selectedSkin") == "Vista-aero" ) {
+     if (Services.prefs.getCharPref("general.skins.selectedSkin") == "Vista-aero" ) {
        gBrowser.selectedTab = newTab;
        gBrowser.updateCurrentBrowser();
      }
@@ -185,10 +165,6 @@ function TMP_BrowserOpenTab(aTab, replaceLastTab) {
        gBrowser.tabContainer.setAttribute("closebuttons", "noclose");
        gBrowser.tabContainer.removeAttribute("closebuttons-hover");
      }
-     // fix a bug in Firefox when we replace last tab the strip remain visible even if the pref is to hide tabbar
-     // when there is only one tab
-     if (!Tabmix.isVersion(40) && TabmixSvc.prefs.getBoolPref("browser.tabs.autoHide") && gBrowser.getStripVisibility())
-       gBrowser.setStripVisibilityTo(false);
    }
 
    if (!loadBlank) {
@@ -197,36 +173,41 @@ function TMP_BrowserOpenTab(aTab, replaceLastTab) {
          browser.stop();
          let originCharset = gBrowser.contentDocument.characterSet;
          browser.loadURIWithFlags(url, flags, null, originCharset);
-         if (Tabmix.isVersion(36))
-            gBrowser.selectedBrowser.focus();
-         else
-            focusElement(content);
+         gBrowser.selectedBrowser.focus();
       }
       catch (ex) {}
    }
    if (aTab && aTab.localName == "tab")
-      gBrowser.TMmoveTabTo(newTab, aTab._tPos + 1);
-   else if (!replaceLastTab && TabmixSvc.TMPprefs.getBoolPref("openNewTabNext")) {
-      gBrowser.TMmoveTabTo(newTab, gBrowser.mCurrentTab._tPos + gBrowser.tabContainer.nextTab, 1);
-      if (TabmixSvc.TMPprefs.getBoolPref("openTabNextInverse"))
-         gBrowser.tabContainer.nextTab++;
+      gBrowser.moveTabTo(newTab, aTab._tPos + 1);
+   else if (!replaceLastTab && Tabmix.prefs.getBoolPref("openNewTabNext")) {
+      // we used to move tab after lastRelatedTab but we don't need it on new tabs
+      // and it mess with recently used tabs order
+      gBrowser.moveTabTo(newTab, gBrowser.selectedTab._tPos + 1);
    }
 
    // always select new tab when replacing last tab
-   var loadInBackground  = replaceLastTab ? false : TabmixSvc.TMPprefs.getBoolPref("loadNewInBackground");
+   var loadInBackground  = replaceLastTab ? false : Tabmix.prefs.getBoolPref("loadNewInBackground");
    gBrowser.TMP_selectNewForegroundTab(newTab, loadInBackground);
+   // make sure to update recently used tabs
+   // if user open many tabs quickly select event don't have time to fire
+   // before new tab select
+   if (!loadInBackground)
+     TMP_LastTab.PushSelectedTab();
 
    // focus the address bar on new tab
-   if (TabmixSvc.TMPprefs.getBoolPref("selectLocationBar") || loadBlank)
-     TMP_clearUrlBar(newTab, url);
+   var clearUrlBar = !replaceLastTab && Tabmix.prefs.getBoolPref("selectLocationBar") ||
+       replaceLastTab && Tabmix.prefs.getBoolPref("selectLocationBar.afterLastTabClosed") ||
+       loadBlank;
+   if (clearUrlBar)
+     Tabmix.clearUrlBar(newTab, url);
 
    return newTab;
 }
 
-function TMP_clearUrlBar(aTab, aUrl, aTimeOut) {
+Tabmix.clearUrlBar = function TMP_clearUrlBar(aTab, aUrl, aTimeOut) {
   if(/about:home|(www\.)*(google|bing)\./.test(aUrl))
     return;
-  if (aUrl != "about:blank") {
+  if (!Tabmix.isBlankPageURL(aUrl)) {
     // clean the the address bar as if the user laod about:blank tab
     gBrowser.tabmix_tab = aTab;
     gBrowser.tabmix_userTypedValue = aUrl;
@@ -242,32 +223,34 @@ function TMP_clearUrlBar(aTab, aUrl, aTimeOut) {
 
 /**
  * @brief In TMP_BrowserOpenTab we empty and fucos the urlbar
- *        if the user or onload from a page blur the urlbar befroe user typed new valur
+ *        if the user or onload from a page blur the urlbar befroe user typed new value
  *        we restore the current url
  */
-function TMP_urlBarOnBlur() {
+Tabmix.urlBarOnBlur = function TMP_urlBarOnBlur() {
+  if (Tabmix.isBlankPageURL(gURLBar.value))
+    gURLBar.value = "";
   if (!gBrowser.tabmix_tab)
     return;
 
   var isCurrentTab = gBrowser.tabmix_tab == gBrowser.mCurrentTab;
   var browser = gBrowser.getBrowserForTab(gBrowser.tabmix_tab);
   var url = gBrowser.tabmix_userTypedValue;
-  if (url != "about:blank")
+  if (!Tabmix.isBlankPageURL(url))
     browser.userTypedValue = url;
   if (isCurrentTab && gBrowser.mIsBusy) {
     browser.addEventListener("load", function TMP_onLoad_urlBarOnBlur(aEvent) {
       aEvent.currentTarget.removeEventListener("load", TMP_onLoad_urlBarOnBlur, true);
-      TMP_updateUrlBarValue();
+      Tabmix.updateUrlBarValue();
     }, true);
     return;
   }
 
-  TMP_updateUrlBarValue()
+  this.updateUrlBarValue()
 }
 
-function TMP_updateUrlBarValue() {
+Tabmix.updateUrlBarValue = function TMP_updateUrlBarValue() {
   var url = gBrowser.currentURI.spec;
-  if (url != gURLBar.value && url != "about:blank") {
+  if (url != gURLBar.value && !Tabmix.isBlankPageURL(url)) {
     gURLBar.value = gBrowser.userTypedValue = url;
   }
   delete gBrowser.tabmix_tab;
@@ -278,30 +261,29 @@ function TMP_updateUrlBarValue() {
  * @brief Load URLs from the URL bar.
  *
  * @param event         A valid event union.
- * @param aPostData     Additional opaque data used by __TMP_LoadBarURL().
+ * @param aPostData     Additional opaque data used by Tabmix.__loadURLBar().
  * @param altDisabled   parameter set by URL Suffix extension, to prevent ALT from opening new tab
- * @param aUrl          aUrl , not in use anymore  
- *                      in the past we used this arg from PopupAutoCompleteRichResult.onPopupClick
+ * @param aUrl          aUrl , url to load.
  * @param mayInheritPrincipal
                         when false prevent any loads from inheriting the currently loaded document's principal
  * @return              Nothing.
  *
  *
  * we call this function from urlbar.handleCommand up to Firefox 10
- * and from extensions.js for objURLsuffix.  
+ * and from extensions.js for objURLsuffix.
  *
  */
-function TMP_BrowserLoadURL(theEvent, aPostData, altDisabled, aUrl, mayInheritPrincipal) {
-  var newTabPref = TabmixSvc.TMPprefs.getBoolPref("opentabfor.urlbar");
-  var theBGPref  = TabmixSvc.TMPprefs.getBoolPref("loadUrlInBackground");
+Tabmix.browserLoadURL = function TMP_BrowserLoadURL(theEvent, aPostData, altDisabled, aUrl, mayInheritPrincipal) {
+  var newTabPref = Tabmix.prefs.getBoolPref("opentabfor.urlbar");
+  var theBGPref  = Tabmix.prefs.getBoolPref("loadUrlInBackground");
   var theURI = aUrl || gURLBar.value;
   if ("gIeTab" in window)
      theURI = gIeTab.getHandledURL(theURI, gURLBar.isModeIE);
 
   var middleClick = theEvent instanceof MouseEvent && (theEvent.button == 1 || theEvent.ctrlKey || theEvent.metaKey);
-  if (aUrl || middleClick) {
+  if (middleClick) {
     let where = whereToOpenLink(theEvent);
-    if (Tabmix.isVersion(40) && where != "current") {
+    if (where != "current") {
       gURLBar.handleRevert();
       gBrowser.selectedBrowser.focus();
     }
@@ -315,8 +297,8 @@ function TMP_BrowserLoadURL(theEvent, aPostData, altDisabled, aUrl, mayInheritPr
     }
   }
   var isAltKey = !altDisabled && (theEvent instanceof Event && ('altKey' in theEvent && theEvent.altKey));
-  newTabPref = TMP_whereToOpen(newTabPref, isAltKey).inNew;
-  __TMP_LoadBarURL(theURI, theEvent, newTabPref, theBGPref, aPostData, true, mayInheritPrincipal);
+  newTabPref = this.whereToOpen(newTabPref, isAltKey).inNew;
+  this.__loadURLBar(theURI, theEvent, newTabPref, theBGPref, aPostData, true, mayInheritPrincipal);
 }
 
 /**
@@ -337,7 +319,7 @@ function TMP_BrowserLoadURL(theEvent, aPostData, altDisabled, aUrl, mayInheritPr
  * @returns                       Nothing.
  *
  */
-function __TMP_LoadBarURL(aURI, aEvent, aNewTabPref, aLoadInBackground, aPostData, aAllowThirdPartyFixup, mayInheritPrincipal) {
+Tabmix.__loadURLBar = function __TMP_LoadBarURL(aURI, aEvent, aNewTabPref, aLoadInBackground, aPostData, aAllowThirdPartyFixup, mayInheritPrincipal) {
   var originCharset = null;
 
   if (gBrowser.localName != "tabbrowser") {
@@ -379,13 +361,10 @@ function __TMP_LoadBarURL(aURI, aEvent, aNewTabPref, aLoadInBackground, aPostDat
       flags |= Ci.nsIWebNavigation.LOAD_FLAGS_DISALLOW_INHERIT_OWNER;
     }
     gBrowser.loadURIWithFlags(aURI, flags, null, null, aPostData);
-    gBrowser.tabContainer.ensureTabIsVisible(gBrowser.mCurrentTab._tPos);
+    gBrowser.tabContainer.mTabstrip.ensureElementIsVisible(gBrowser.selectedTab);
   }
 
-  if (Tabmix.isVersion(36))
-    gBrowser.selectedBrowser.focus();
-  else
-    focusElement(content);
+  gBrowser.selectedBrowser.focus();
 
   if (aEvent instanceof Event) {
     aEvent.preventDefault();
@@ -394,23 +373,27 @@ function __TMP_LoadBarURL(aURI, aEvent, aNewTabPref, aLoadInBackground, aPostDat
   return;
 }
 
-/* call from TMP_TBP_init and from text.link.xul */
-function TMP_openUILink_init() {
+/**
+ * @brief openUILink handles clicks on UI elements that cause URLs to load
+ *
+ * called from Tabmix.linkHandling_init and from text.link.xul
+ *
+ */
+Tabmix.openUILink_init = function TMP_openUILink_init() {
   if ("openUILink" in window) {
-    Tabmix.newCode("openUILink", openUILink)._replace(
-      'var where = whereToOpenLink(e, ignoreButton, ignoreAlt);',
-      <![CDATA[$&
-        var win = Tabmix.getTopWin();
-        if (win) {
-          // don't open blanke tab when we are about to add new livemark
-          let _addLivemark = /^feed:/.test(url) && TabmixSvc.prefs.getCharPref("browser.feeds.handler") == "bookmarks";
-          if (where == "current" && !_addLivemark)
-            where = win.TMP_checkCurrent(url);
+    this.newCode("openUILink", openUILink)._replace(
+      /(openUILinkIn[^\(]*\([^\)]+)(\))/,
+      <![CDATA[
+        var win = getTopWin();
+        if (win && where == "current") {
+          // don't open blank tab when we are about to add new livemark
+          let _addLivemark = /^feed:/.test(url) &&
+             Services.prefs.getCharPref("browser.feeds.handler") == "bookmarks";
+          if (!_addLivemark)
+            where = win.Tabmix.checkCurrent(url);
         }
+        try {$&}  catch (ex) {  }
       ]]>
-    )._replace(
-      'openUILinkIn(url, where, allowKeywordFixup, postData, referrerUrl);',
-      'try {$&} catch (ex) {  }'
     )._replace( // fix incompatibility with Omnibar (O is not defined)
       'O.handleSearchQuery',
       'window.Omnibar.handleSearchQuery', {silent: true}
@@ -418,33 +401,39 @@ function TMP_openUILink_init() {
   }
 }
 
-function TMP_checkCurrent(url) {
-  if (gBrowser.mCurrentTab.hasAttribute("locked")) {
-    var isBlankTab = gBrowser.isBlankNotBusyTab(gBrowser.mCurrentTab);
+Tabmix.checkCurrent = function TMP_checkCurrent(url) {
+  var opentabforLinks = Tabmix.prefs.getIntPref("opentabforLinks");
+  if (opentabforLinks == 1 || gBrowser.mCurrentTab.hasAttribute("locked")) {
+    let isBlankTab = gBrowser.isBlankNotBusyTab(gBrowser.mCurrentTab);
     if (!isBlankTab)
        return "tab";
   }
-  else if (TabmixSvc.prefs.getIntPref("extensions.tabmix.opentabforLinks") == 2 ) {
+  else if (opentabforLinks == 2) {
     // Get current page url
-    var curpage = gBrowser.currentURI.spec;
-    var domain = TMP_checkDomain(curpage, url);
+    let curpage = gBrowser.currentURI.spec;
+    let domain = this.contentAreaClick.checkDomain(curpage, url);
     if (domain.current && domain.target && domain.target != domain.current)
       return "tab";
   }
   return "current";
 }
 
-function TMP_copyTabData(newTab, oldTab) {
+/**
+ * @brief copy Tabmix data from old tab to new tab.
+ *        we use it before swapBrowsersAndCloseOther
+ */
+Tabmix.copyTabData = function TMP_copyTabData(newTab, oldTab) {
   let _xulAttributes = ["protected", "_locked", "fixed-label", "label-uri", "reload-data", "tabmix_bookmarkId"];
 
+  var self = this;
   _xulAttributes.forEach(function _setData(attr) {
-    Tabmix.setItem(newTab, attr, oldTab.hasAttribute(attr) ? oldTab.getAttribute(attr) : null);
+    self.setItem(newTab, attr, oldTab.hasAttribute(attr) ? oldTab.getAttribute(attr) : null);
   });
 
-  TMP_restoreTabState(newTab);
+  this.restoreTabState(newTab);
 }
 
-function TMP_restoreTabState(aTab) {
+Tabmix.restoreTabState = function TMP_restoreTabState(aTab) {
   if (aTab.hasAttribute("_locked")) {
     if (aTab.getAttribute("_locked") == "true")
       aTab.setAttribute("locked", "true");
@@ -455,7 +444,7 @@ function TMP_restoreTabState(aTab) {
   // this function run before tab load, so onTabReloaded will run when onStateChange get STATE_STOP
   var reloadData = aTab.getAttribute("reload-data");
   if (reloadData) {
-    Tabmix.autoReload.initTab(aTab);
+    this.autoReload.initTab(aTab);
     aTab.autoReloadEnabled = true;
     aTab.setAttribute("_reload", true);
     reloadData = reloadData.split(" ");
@@ -465,11 +454,11 @@ function TMP_restoreTabState(aTab) {
 
   if (aTab.hasAttribute("tabmix_bookmarkId")) {
     // make sure the id exist before using it
+    let bmitemid = aTab.getAttribute("tabmix_bookmarkId");
     try {
-      let _URI = PlacesUtils.bookmarks.getBookmarkURI(bmitemid);
-      let title = _URI && _URI.spec == aUrl && PlacesUtils.bookmarks.getItemTitle(bmitemid);
+      let title = PlacesUtils.bookmarks.getItemTitle(bmitemid);
     } catch (ex) {
-      aTab.removeAttribute("tabmix_bookmarkId")
+      aTab.removeAttribute("tabmix_bookmarkId");
     }
   }
-}
\ No newline at end of file
+}
