@@ -28,22 +28,25 @@ var TMP_SessionStore = {
      let selectedTab = aUndoItem.tabs[aUndoItem.selected - 1];
      if (!selectedTab.entries || selectedTab.entries.length == 0)
        return;
-     let activeIndex = (selectedTab.index || selectedTab.entries.length) - 1;
-     if (activeIndex >= selectedTab.entries.length)
-       activeIndex = selectedTab.entries.length - 1;
-     let tabData = selectedTab.entries[activeIndex];
+     let tabData = this.getActiveEntryData(selectedTab);
      let url = selectedTab.attributes["label-uri"];
      if (url == tabData.url || url == "*")
        aUndoItem.title = selectedTab.attributes["fixed-label"];
      else {
-       aUndoItem.title = TMP_getTitleFromBookmark(tabData.url, aUndoItem.title || tabData.title || tabData.url);
-       if (aUndoItem.title == "about:blank") {
-         let string = Tabmix.isVersion(40) ?  "tabs.emptyTabTitle" : "tabs.untitled";
-         aUndoItem.title = gBrowser.mStringBundle.getString(string);
-       }
+       aUndoItem.title = TMP_Places.getTitleFromBookmark(tabData.url, aUndoItem.title || tabData.title || tabData.url);
+       if (aUndoItem.title == "about:blank")
+         aUndoItem.title = gBrowser.mStringBundle.getString("tabs.emptyTabTitle");
      }
    },
 
+   // get nsSessionStore active entry data.
+   getActiveEntryData: function TMP_ss_getActiveEntryData(aData) {
+     let activeIndex = (aData.index || aData.entries.length) - 1;
+     if (activeIndex >= aData.entries.length)
+       activeIndex = aData.entries.length - 1;
+     return aData.entries[activeIndex] || {};
+   },
+
    /**
     * @brief       - Add attribute to nsSessionStore persistTabAttribute.
     *
@@ -93,7 +96,7 @@ var TMP_SessionStore = {
     * @returns       Nothing.
     */
    setService: function TMP_ss_setSessionService(msgNo, start, win) {
-      if ("tabmix_setSession" in window || TabmixSvc.prefs.prefHasUserValue("extensions.tabmix.setDefault"))
+      if ("tabmix_setSession" in window || Tabmix.prefs.prefHasUserValue("setDefault"))
          return;
      /*
       * From 2008-03-10 we don't set browser.sessionstore.enabled to false anymore
@@ -107,8 +110,8 @@ var TMP_SessionStore = {
       */
       const TMP_SS_MANAGER = "extensions.tabmix.sessions.manager";
       const TMP_SS_CRASHRECOVERY = "extensions.tabmix.sessions.crashRecovery";
-      var TMP_manager_enabled = TabmixSvc.prefs.getBoolPref(TMP_SS_MANAGER);
-      var TMP_crashRecovery_enabled = TabmixSvc.prefs.getBoolPref(TMP_SS_CRASHRECOVERY);
+      var TMP_manager_enabled = Services.prefs.getBoolPref(TMP_SS_MANAGER);
+      var TMP_crashRecovery_enabled = Services.prefs.getBoolPref(TMP_SS_CRASHRECOVERY);
       if (!TMP_manager_enabled && !TMP_crashRecovery_enabled) {
          return;
       }
@@ -118,41 +121,41 @@ var TMP_SessionStore = {
       if (Tabmix.extensions.sessionManager) {
          // update session manager settings accourding to current tabmix settings
          if (TMP_manager_enabled) {
-            TabmixSvc.prefs.setBoolPref(TMP_SS_MANAGER, false);
-            switch (TabmixSvc.SMprefs.getIntPref("onStart")) {
+            Services.prefs.setBoolPref(TMP_SS_MANAGER, false);
+            switch (Tabmix.prefs.getIntPref("sessions.onStart")) {
                case 0:
-                  TabmixSvc.prefs.setIntPref("extensions.sessionmanager.startup", 0);
-                  TabmixSvc.prefs.setIntPref("browser.startup.page", 3);
+                  Services.prefs.setIntPref("extensions.sessionmanager.startup", 0);
+                  Services.prefs.setIntPref("browser.startup.page", 3);
                   break;
                case 1:
-                  TabmixSvc.prefs.setIntPref("extensions.sessionmanager.startup", 1);
+                  Services.prefs.setIntPref("extensions.sessionmanager.startup", 1);
                   break;
                //default: nothing to do
             }
-            switch (TabmixSvc.SMprefs.getIntPref("onClose")) {
+            switch (Tabmix.prefs.getIntPref("sessions.onClose")) {
                case 0:
-                  TabmixSvc.prefs.setIntPref("extensions.sessionmanager.backup_session", 1);
+                  Services.prefs.setIntPref("extensions.sessionmanager.backup_session", 1);
                   break;
                case 1:
-                  TabmixSvc.prefs.setIntPref("extensions.sessionmanager.backup_session", 2);
+                  Services.prefs.setIntPref("extensions.sessionmanager.backup_session", 2);
                   break;
                default:
-                  TabmixSvc.prefs.setIntPref("extensions.sessionmanager.backup_session", 0);
+                  Services.prefs.setIntPref("extensions.sessionmanager.backup_session", 0);
             }
          }
          if (TMP_crashRecovery_enabled) {
-            TabmixSvc.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
-            TabmixSvc.prefs.setBoolPref("browser.sessionstore.resume_from_crash", true);
+            Services.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
+            Services.prefs.setBoolPref("browser.sessionstore.resume_from_crash", true);
          }
          delete window.tabmix_setSession;
       }
       else if (this.isSessionStoreEnabled()) {
          // ask the user to choose between TMP session manager and sessionstore
          // we use non modal promptService on start up, so we disabled Tabmix session managerto let the startup
-         // process continue and set tothe appropriate after the dialog prompt dismissed.
+         // process continue and set the appropriate preference after the dialog prompt dismissed.
          if (start) {
-            TabmixSvc.prefs.setBoolPref(TMP_SS_MANAGER, false);
-            TabmixSvc.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
+            Services.prefs.setBoolPref(TMP_SS_MANAGER, false);
+            Services.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
          }
          let title = "TabMix " + TabmixSvc.getSMString("sm.title");
          let msg = start ? TabmixSvc.getSMString("sm.disable.msg") + "\n\n" : "";
@@ -165,34 +168,34 @@ var TMP_SessionStore = {
          let callBack = function (aResult) {
             if ((msgNo == 1 && aResult.button == 1) || ((msgNo == 2 && aResult.button == 0))) {
               self.setSessionRestore(false);
-              TabmixSvc.prefs.setBoolPref(TMP_SS_MANAGER, TMP_manager_enabled);
-              TabmixSvc.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, TMP_crashRecovery_enabled);
+              Services.prefs.setBoolPref(TMP_SS_MANAGER, TMP_manager_enabled);
+              Services.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, TMP_crashRecovery_enabled);
             }
             else {
               // we don't change any of sessionstore default setting
               // the user will be ask on exit what to do. (browser.warnOnRestart and browser.warnOnQuit are both true on default)
-              TabmixSvc.prefs.setBoolPref(TMP_SS_MANAGER, false);
-              TabmixSvc.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
+              Services.prefs.setBoolPref(TMP_SS_MANAGER, false);
+              Services.prefs.setBoolPref(TMP_SS_CRASHRECOVERY, false);
            }
            delete window.tabmix_setSession;
          }
-         let result = Tabmix.promptService([TMP_BUTTON_OK, TMP_HIDE_MENUANDTEXT, TMP_HIDE_CHECKBOX],
+         let result = Tabmix.promptService([Tabmix.BUTTON_OK, Tabmix.HIDE_MENUANDTEXT, Tabmix.HIDE_CHECKBOX],
                [title, msg, "", "", buttons], win || window, start ? callBack : null);
          if (!start)
            callBack(result);
       }
       // when user start new profile or update from firefox 2.0 profile browser.warnOnRestart and browser.warnOnQuit are both true on default
-      else if (!TabmixSvc.prefs.prefHasUserValue("browser.warnOnRestart") ||
-                !TabmixSvc.prefs.prefHasUserValue("browser.warnOnQuit ")) {
-         TabmixSvc.prefs.setBoolPref("browser.warnOnRestart", false);
-         TabmixSvc.prefs.setBoolPref("browser.warnOnQuit", false);
+      else if (!Services.prefs.prefHasUserValue("browser.warnOnRestart") ||
+                !Services.prefs.prefHasUserValue("browser.warnOnQuit ")) {
+         Services.prefs.setBoolPref("browser.warnOnRestart", false);
+         Services.prefs.setBoolPref("browser.warnOnQuit", false);
          delete window.tabmix_setSession;
       }
    },
 
    isSessionStoreEnabled: function () {
-     return TabmixSvc.prefs.getIntPref("browser.startup.page") == 3 ||
-            TabmixSvc.prefs.getBoolPref("browser.sessionstore.resume_from_crash");
+     return Services.prefs.getIntPref("browser.startup.page") == 3 ||
+            Services.prefs.getBoolPref("browser.sessionstore.resume_from_crash");
    },
 
    afterSwitchThemes: false,
@@ -202,10 +205,10 @@ var TMP_SessionStore = {
       if (!Tabmix.isFirstWindow)
          return false;
 
-      // in Firefox 4.0+ when we close all browser windows without exit (non browser windows are opened)
-      // Firefox restore next browser window that open
+      // When we close all browser windows without exit (non browser windows are opened)
+      // Firefox restore current session when a browser window opens
       if (Tabmix.numberOfWindows(false, null) > 1)
-         return Tabmix.isVersion(40);
+         return true;
 
       var ss = Cc["@mozilla.org/browser/sessionstartup;1"].
                     getService(Ci.nsISessionStartup);
@@ -214,13 +217,13 @@ var TMP_SessionStore = {
    },
 
    setSessionRestore: function (aEnable) {
-      TabmixSvc.prefs.setBoolPref("browser.warnOnRestart", aEnable);
-      TabmixSvc.prefs.setBoolPref("browser.warnOnQuit", aEnable);
-      TabmixSvc.prefs.setBoolPref("browser.sessionstore.resume_from_crash", aEnable);
+      Services.prefs.setBoolPref("browser.warnOnRestart", aEnable);
+      Services.prefs.setBoolPref("browser.warnOnQuit", aEnable);
+      Services.prefs.setBoolPref("browser.sessionstore.resume_from_crash", aEnable);
       if (aEnable)
-        TabmixSvc.prefs.setIntPref("browser.startup.page", 3);
-      else if (TabmixSvc.prefs.getIntPref("browser.startup.page") == 3)
-        TabmixSvc.prefs.setIntPref("browser.startup.page", 1);
+        Services.prefs.setIntPref("browser.startup.page", 3);
+      else if (Services.prefs.getIntPref("browser.startup.page") == 3)
+        Services.prefs.setIntPref("browser.startup.page", 1);
    },
 
   /**
@@ -230,7 +233,7 @@ var TMP_SessionStore = {
    *
    * @param aUri      string value - url address
    *
-   * @param aTitle      string value - title
+   * @param aTitle    string value - title
    *
    * @returns         tab title - string.
    */
@@ -239,7 +242,7 @@ var TMP_SessionStore = {
       if (fixedLabelUri == aUri || fixedLabelUri == "*")
          return this._getAttribute(aData, "fixed-label");
 
-      return TMP_getTitleFromBookmark(aUri, aTitle, this._getAttribute(aData, "tabmix_bookmarkId"));
+      return TMP_Places.getTitleFromBookmark(aUri, aTitle, this._getAttribute(aData, "tabmix_bookmarkId"));
    },
 
    /**
@@ -252,7 +255,6 @@ var TMP_SessionStore = {
     * @returns         attribute value as string or empty string.
     */
    _getAttribute: function TMP_ss__getAttribute(aTabData, attrib) {
-      // tabData.attributes is in use for Firefox 3.5+
       if (aTabData.attributes && attrib in aTabData.attributes)
          return aTabData.attributes[attrib];
 
@@ -307,9 +309,7 @@ var TMP_ClosedTabs = {
    },
 
    getUrl: function ct_getUrl(aTabData) {
-      var history = aTabData.state;
-      var activeIndex = (history.index || history.entries.length) - 1;
-      return history.entries[activeIndex].url;
+      return TMP_SessionStore.getActiveEntryData(aTabData.state).url;
    },
 
    getTitle: function ct_getTitle(aTabData, aUri) {
@@ -322,7 +322,7 @@ var TMP_ClosedTabs = {
       if (TabmixAllTabs.isAfterCtrlClick(aPopup.parentNode))
          return false;
 
-      TabmixAllTabs.beforeCommonList(aPopup);
+      TabmixAllTabs.beforeCommonList(aPopup, true);
 
       // populate menu
       var closedTabs = this.getClosedTabData;
@@ -374,23 +374,13 @@ var TMP_ClosedTabs = {
       m = aPopup.appendChild(document.createElement("menuitem"));
       m.setAttribute("id", "clearClosedTabsList");
       m.setAttribute("label", TabmixSvc.getString("undoclosetab.clear.label"));
-      if (!Tabmix.isVersion(40))
-        m.setAttribute("accesskey", TabmixSvc.getString("undoclosetab.clear.accesskey"));
       m.setAttribute("value", -1);
       m.setAttribute("oncommand", "TMP_ClosedTabs.restoreTab('original', -1); event.stopPropagation();");
 
       // "Restore All Tabs"
       m = aPopup.appendChild(document.createElement("menuitem"));
       m.setAttribute("id", "restoreAllClosedTabs");
-      if (Tabmix.isVersion(36)) {
-        m.setAttribute("label", gNavigatorBundle.getString("menuRestoreAllTabs.label"));
-        if (!Tabmix.isVersion(40))
-          m.setAttribute("accesskey", gNavigatorBundle.getString("menuRestoreAllTabs.accesskey"));
-      }
-      else {
-        m.setAttribute("label", gNavigatorBundle.getString("menuOpenAllInTabs.label"));
-        m.setAttribute("accesskey", gNavigatorBundle.getString("menuOpenAllInTabs.accesskey"));
-      }
+      m.setAttribute("label", gNavigatorBundle.getString("menuRestoreAllTabs.label"));
       m.setAttribute("value", -2);
       m.setAttribute("oncommand", "TMP_ClosedTabs.restoreTab('original', -2); event.stopPropagation();");
       return true;
@@ -405,7 +395,7 @@ var TMP_ClosedTabs = {
       if (index < 0)
          return;
 
-      var where = TabmixSvc.TMPprefs.getBoolPref("middleclickDelete") ? 'delete' : 'tab';
+      var where = Tabmix.prefs.getBoolPref("middleclickDelete") ? 'delete' : 'tab';
       TMP_ClosedTabs.restoreTab(where, index);
       var popup = aEvent.originalTarget.parentNode;
       if (TMP_ClosedTabs.count > 0)
@@ -467,7 +457,7 @@ var TMP_ClosedTabs = {
    */
    getClosedTabAtIndex: function ct_getClosedTabAtIndex(aIndex) {
       // update our session data
-      var updateRDF = TabmixSessionManager.enableBackup && TabmixSvc.SMprefs.getBoolPref("save.closedtabs");
+      var updateRDF = TabmixSessionManager.enableBackup && Tabmix.prefs.getBoolPref("sessions.save.closedtabs");
       if (updateRDF) {
         if (aIndex >= 0)
            TabmixSessionManager.deleteClosedtabAt(this.count - aIndex);
@@ -488,7 +478,7 @@ var TMP_ClosedTabs = {
 
       // replace existing _closedTabs
       try {
-        TabmixSvc.ss.setWindowState(window, Tabmix.isVersion(40) ? Tabmix.JSON.stringify(state) : state.toSource(), false);
+        TabmixSvc.ss.setWindowState(window, Tabmix.JSON.stringify(state), false);
       } catch (e) {}
 
       this.setButtonDisableState();
@@ -519,7 +509,7 @@ var TMP_ClosedTabs = {
       var multiple = closedTabCount > 1;
       for (i = 0; i < closedTabCount; i++) {
          blankTab = blankTabs.pop();
-         this.SSS_undoCloseTab(0, "original", i==0, blankTab, Tabmix.isVersion(40) && multiple);
+         this.SSS_undoCloseTab(0, "original", i==0, blankTab, multiple);
       }
 
       // remove unused blank tabs
@@ -528,12 +518,10 @@ var TMP_ClosedTabs = {
          blankTab.collapsed = true;
          gBrowser.removeTab(blankTab);
       }
-
-      gBrowser.tabContainer.nextTab = 1;
    },
 
    SSS_undoCloseTab: function ct_SSS_undoCloseTab(aIndex, aWhere, aSelectRestoredTab, aTabToRemove, skipAnimation) {
-      if (!TabmixSvc.TMPprefs.getBoolPref("undoClose") || this.count == 0)
+      if (!Tabmix.prefs.getBoolPref("undoClose") || this.count == 0)
          return null;
 
       // get tab data
@@ -546,8 +534,7 @@ var TMP_ClosedTabs = {
       else if (typeof(aTabToRemove) == "undefined" && gBrowser.isBlankNotBusyTab(cTab))
          aTabToRemove = cTab;
 
-      if ("TabView" in window)
-        TabView.prepareUndoCloseTab(aTabToRemove);
+      TabView.prepareUndoCloseTab(aTabToRemove);
 
       if (aTabToRemove)
          aTabToRemove.collapsed = true;
@@ -559,25 +546,25 @@ var TMP_ClosedTabs = {
       if (aTabToRemove)
          gBrowser.removeTab(aTabToRemove);
       // add restored tab to current window
-      TabmixSvc.ss.setTabState(newTab, Tabmix.isVersion(40) ? Tabmix.JSON.stringify(tabData.state) : tabData.state.toSource());
+      TabmixSvc.ss.setTabState(newTab, Tabmix.JSON.stringify(tabData.state));
 
-      if ("TabView" in window)
-        TabView.afterUndoCloseTab();
+      TabView.afterUndoCloseTab();
 
       // after we open new tab we only need to fix position if this is true
-      var restorePosition = TabmixSvc.TMPprefs.getBoolPref("undoClosePosition");
+      // we don't call moveTabTo from add tab if it called from sss_undoCloseTab
+      var restorePosition = Tabmix.prefs.getBoolPref("undoClosePosition");
       if ( aWhere == "current" || (aWhere == "original" && restorePosition) ) {
-         gBrowser.TMmoveTabTo(newTab, Math.min(gBrowser.tabs.length - 1, tabData.pos), 1);
+         gBrowser.moveTabTo(newTab, Math.min(gBrowser.tabs.length - 1, tabData.pos));
+      }
+      else if (aWhere != "end" && Tabmix.getOpenTabNextPref()) {
+         let newTabPos = (gBrowser._lastRelatedTab || gBrowser.selectedTab)._tPos + 1;
+         gBrowser.moveTabTo(newTab, newTabPos);
       }
-      else if (aWhere != "end" && TMP_getOpenTabNextPref()) // middle click on History > recently closed tabs
-         // we don't call TMP_openTabNext from add tab if it called from sss_undoCloseTab
-         gBrowser.TMP_openTabNext(newTab);
 
       if (aSelectRestoredTab) {
          content.focus();
          gBrowser.TMP_selectNewForegroundTab(newTab, false, null, false);
       }
-
       return newTab;
    },
 
@@ -598,7 +585,7 @@ var TabmixConvertSession = {
 
    startup: function cs_startup() {
       if (!Tabmix.extensions.sessionManager || "tabmix_afterTabduplicated" in window || !Tabmix.isFirstWindow)
-         return;
+        return;
 
       var sessions = TabmixSessionManager.getSessionList();
       if (!sessions)
@@ -610,9 +597,9 @@ var TabmixConvertSession = {
       TabmixSessionManager.setLiteral("rdf:gSessionManager", "status", "converted");
       TabmixSessionManager.saveStateDelayed();
       var callBack = function (aResult) {
-                  if (aResult.button == TMP_BUTTON_OK) {
+                  if (aResult.button == Tabmix.BUTTON_OK) {
                     setTimeout(function (a,b) {
-                      com.morac.gSessionManagerWindowObject.doTMPConvertFile(a,b);
+                      TabmixConvertSession.convertFile(a, b);
                     }, 0, null, true);
                   }
                  }
@@ -635,8 +622,11 @@ var TabmixConvertSession = {
       this.convertFile(fp.fileURL.spec);
    },
 
-   convertFile: function cs_convertFile(aFileUri) {
-      com.morac.gSessionManagerWindowObject.doTMPConvertFile(aFileUri);
+   convertFile: function cs_convertFile(aFileUri, aSilent) {
+      if (typeof com.morac.SessionManagerAddon == "object")
+        com.morac.SessionManagerAddon.gSessionManagerWindowObject.doTMPConvertFile(aFileUri, aSilent);
+      else
+        com.morac.gSessionManagerWindowObject.doTMPConvertFile(aFileUri, aSilent);
    },
 
    confirm: function cs_confirm(aMsg, aCallBack) {
@@ -644,7 +634,7 @@ var TabmixConvertSession = {
                        getService(Ci.nsIStringBundleService);
       let bundle = bunService.createBundle("chrome://global/locale/commonDialogs.properties");
       let buttons = [bundle.GetStringFromName("Yes"), bundle.GetStringFromName("No")].join("\n");
-      return Tabmix.promptService([TMP_BUTTON_OK, TMP_HIDE_MENUANDTEXT, TMP_HIDE_CHECKBOX],
+      return Tabmix.promptService([Tabmix.BUTTON_OK, Tabmix.HIDE_MENUANDTEXT, Tabmix.HIDE_CHECKBOX],
             [this.getTitle, aMsg, "", "", buttons], window, aCallBack);
    },
 
@@ -681,17 +671,15 @@ var TabmixConvertSession = {
       state.sizemode = (window.windowState == window.STATE_MAXIMIZED) ? "maximized" : "normal";
 
       // save panorama data if exist
-      if ("TabView" in window) {
-        let extData = {};
-        let emptyGroup = Tabmix.JSON.stringify({})
-        extData["tabview-groups"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-groups", emptyGroup);
-        extData["tabview-group"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-group", emptyGroup);
-        extData["tabview-ui"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-ui", emptyGroup);
-        extData["tabview-visibility"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-visibility", false);
-        if (Tabmix.isVersion(70))
-          extData["tabview-last-session-group-name"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-last-session-group-name", "");
-        state.extData = extData;
-      }
+      let extData = {};
+      let emptyGroup = Tabmix.JSON.stringify({})
+      extData["tabview-groups"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-groups", emptyGroup);
+      extData["tabview-group"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-group", emptyGroup);
+      extData["tabview-ui"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-ui", emptyGroup);
+      extData["tabview-visibility"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-visibility", false);
+      if (Tabmix.isVersion(70))
+        extData["tabview-last-session-group-name"] = TabmixSessionManager.getLiteralValue(rdfNodeWindow, "tabview-last-session-group-name", "");
+      state.extData = extData;
       return state;
    },
 
@@ -767,7 +755,7 @@ var TabmixConvertSession = {
          });
       }
       // save panorama data if exist
-      if ("TabView" in window && !aClosedTab) {
+      if (!aClosedTab) {
         let data = TabmixSessionManager.getLiteralValue(rdfNodeTab, "tabview-tab");
         if (data != "") {
           tabData.extData = {};
