@@ -1,32 +1,10 @@
-/**
- * We replace some global Services with smart getters
- * gIOService     >  TabmixSvc.io
- * gTabmixPrefs   >  TabmixSvc.prefs
- * tabxPrefs      >  TabmixSvc.TMPprefs
- * SessionPref    > TabmixSvc.SMprefs
- * gWindowManager > TabmixSvc.wm
- *
- **/
-
- /**
-rename functions and global variable:
-
-utils.js
-========
-TM_PromptService > Tabmix.promptService
-gIsFirefox35 > Tabmix.isVersion(35)
-gIsFirefox36 > Tabmix.isVersion(36)
-gIsFirefox37 > Tabmix.isVersion(37)
-
-*/
-
 function Tabmix_ChangeCode(aObjectName, aCodeString, aForceUpdate) {
   this.name = aObjectName;
   this.value = aCodeString;
   this.needUpdate = aForceUpdate;
 }
 
-Tabmix_ChangeCode.prototype =  {
+Tabmix_ChangeCode.prototype = {
   needUpdate: false,
   _replace: function TMP_utils__replace(substr ,newString, aParams) {
     var silent;
@@ -51,7 +29,7 @@ Tabmix_ChangeCode.prototype =  {
       this.value = this.value.replace(substr, newString);
       this.needUpdate = true;
     }
-    else if (!silent){
+    else if (!silent) {
       Tabmix.clog(Tabmix.callerName() + " can't find string: " + substr
           + "\nin " + this.name + "."
           + "\n\nTry Tabmix latest development version from tmp.garyr.net/tab_mix_plus-dev-build.xpi,"
@@ -63,36 +41,53 @@ Tabmix_ChangeCode.prototype =  {
 
   toSetter: function(aObj, aName) {
     Tabmix._define("setter", aObj, aName, this.value);
-    delete this;
   },
 
   toGetter: function(aObj, aName) {
     Tabmix._define("getter", aObj, aName, this.value);
-    delete this;
   },
 
   toCode: function TMP_utils_toCode(aShow, aObj, aName) {
-     try {
-       if (this.needUpdate)
-         Tabmix.toCode(aObj, aName || this.name, this.value);
-       if (aShow)
-         this.show();
-       delete this;
+    try {
+      if (Tabmix._debugMode) {
+        this.value = this.value.replace("{", "{try {") +
+            ' catch (ex) {Tabmix.assert(ex, "outer try-catch in ' + (aName || this.name) + '");}}';
+      }
+      if (this.needUpdate)
+        Tabmix.toCode(aObj, aName || this.name, this.value);
+      else if (Tabmix._debugMode)
+        Tabmix.clog("in " + Tabmix.callerName() + " no update needed to " + (aName || this.name));
+      if (aShow)
+        this.show(aObj, aName);
     } catch (ex) {
       Components.utils.reportError("Tabmix " + Tabmix.callerName() + " failed to change " + this.name + "\nError: " + ex.message);
     }
   },
 
-  show: function() {
+  show: function(aObj, aName) {
     if (this.name != null)
       Tabmix.show(this.name);
+    else if (aObj && aName in aObj)
+      Tabmix.clog(aObj[aName].toString());
   }
 
 }
 
 var Tabmix = {
   newCode: function(aObjectName, aObject, aForceUpdate) {
-    return new Tabmix_ChangeCode(aObjectName, aObject.toString(), aForceUpdate);
+    try {
+      return new Tabmix_ChangeCode(aObjectName, aObject.toString(), aForceUpdate);
+    } catch (ex) {
+      this.log("aObjectName " + aObjectName + "\n" + ex);
+      if (Tabmix._debugMode)
+        this.obj(aObject, "aObject");
+    }
+    return null;
+  },
+
+  get prefs() {
+    delete this.prefs;
+    return this.prefs = Services.prefs.getBranch("extensions.tabmix.");
   },
 
   isVersion: function(aVersionNo) {
@@ -104,6 +99,7 @@ var Tabmix = {
   },
 
   getObject: function (aMethod, rootID) {
+try {
     var methodsList = aMethod.split(".");
     if (methodsList[0] == "window")
       methodsList.shift();
@@ -116,6 +112,10 @@ var Tabmix = {
       obj = obj[aFn];
     });
     return obj;
+} catch (ex) {
+this.log(aMethod)
+}
+   return null;
   },
 
   show: function(aMethod, rootID, aDelay) {
@@ -138,21 +138,21 @@ var Tabmix = {
   },
 
   // for debug
-    debug: function TMP_utils_debug(aMessage, aShowCaller) {
+  debug: function TMP_utils_debug(aMessage, aShowCaller) {
     if (this._debug)
       this.log(aMessage, aShowCaller);
   },
 
   clog: function TMP_utils_clog(aMessage) {
-    TabmixSvc.console.logStringMessage("TabMix :\n" + aMessage);
+    Services.console.logStringMessage("TabMix :\n" + aMessage);
   },
 
   log: function TMP_utils_log(aMessage, aShowCaller, offset) {
     offset = !offset ? 0 : 1;
-    let names = this._getNames(aShowCaller ? 2 + offset: 1 + offset);
+    let names = this._getNames(aShowCaller ? 2 + offset : 1 + offset);
     let callerName = names[offset+0];
     let callerCallerName = aShowCaller ? " (caller was " + names[offset+1] + ")" : "";
-    TabmixSvc.console.logStringMessage("TabMix " + callerName + callerCallerName + " :\n" + aMessage);
+    Services.console.logStringMessage("TabMix " + callerName + callerCallerName + " :\n" + aMessage);
   },
 
   // get functions names from Error().stack
@@ -180,13 +180,22 @@ var Tabmix = {
   _getCallerNameByIndex: function TMP_utils_getCallerNameByIndex(aPlace) {
     let stack = Error().stack.split("\n");
     let fn = stack[TabmixSvc.stackOffset + aPlace];
+
     if (fn)
       return this._name(fn);
     return null;
   },
 
+  // Bug 744842 - don't include actual args in error.stack.toString()
+  // since Bug 744842 landed the stack string don't have (arg1, arg2....)
+  // so we can get the name from the start of the string until @
+  get _char() {
+    delete this._char;
+    return this._char = this.isVersion(140) ? "@" : "(";
+  },
+
   _name: function(fn) {
-    let name = fn.substr(0, fn.indexOf("("));
+    let name = fn.substr(0, fn.indexOf(this._char))
     if (!name) {
       // get file name and line number
       let lastIndexOf = fn.lastIndexOf("/");
@@ -195,6 +204,20 @@ var Tabmix = {
     return name;
   },
 
+/*
+  _nameFromComponentsStack: function (Cs) {
+    return Cs.name ||
+           Cs.filename.substr(Cs.filename.lastIndexOf("/") + 1) + ":" + Cs.lineNumber;
+  },
+
+  callerName: function () {
+///    return this._getCallerNameByIndex(2);
+    try {
+      var name = this._nameFromComponentsStack(Components.stack.caller.caller);
+    } catch (ex) { }
+    return name || "";
+  },
+*/
   callerName: function () {
     return this._getCallerNameByIndex(2);
   },
@@ -211,7 +234,6 @@ var Tabmix = {
       let callerName = Components.stack.caller.caller.name;
       if (!callerName)
         return false;
-///this.log("callerName " + callerName +"\narguments " + (typeof arguments[0] == "object" ? arguments[0] : Array.prototype.slice.call(arguments)))
       if (typeof arguments[0] == "object")
         return arguments[0].indexOf(callerName) > -1;
 
@@ -224,6 +246,16 @@ var Tabmix = {
     return false;
   },
 
+///XXX add new arg as options as object
+/*
+options = {
+  msg: msg
+  log: true / false; defaul true
+  function: true / false defaul false
+  deep: true / false defaul false
+  offset; for intenal use only true / false defaul false
+}
+*/
   obj: function(aObj, aMessage, aDisallowLog, level) {
     var offset = typeof level == "string" ? "  " : "";
     aMessage = aMessage ? offset + aMessage + "\n" : "";
@@ -257,13 +289,16 @@ var Tabmix = {
   },
 
   assert: function TMP_utils_assert(aError, aMsg) {
+/*
+XXX fix this when there is no stack go directly to trace
+*/
     let names = this._getNames(1, aError.stack);
     let errAt = " at " + names[0];
     let location = aError.location ? "\n" + aError.location : "";
     let assertionText = "Tabmix Plus ERROR" + errAt + ":\n" + (aMsg ? aMsg + "\n" : "") + aError.message + location;
     let stackText = "stack" in aError ? "\nStack Trace: \n" + aError.stack : "";
     if (stackText)
-      TabmixSvc.console.logStringMessage(assertionText + stackText);
+      Services.console.logStringMessage(assertionText + stackText);
     else
       this.trace(assertionText, 2);
   },
@@ -272,7 +307,7 @@ var Tabmix = {
     // cut off the first line of the stack trace, because that's just this function.
     let stack = Error().stack.split("\n").slice(slice || 1);
 
-    TabmixSvc.console.logStringMessage("Tabmix Trace: " + aMsg + '\n' + stack.join("\n"));
+    Services.console.logStringMessage("Tabmix Trace: " + (aMsg || "") + '\n' + stack.join("\n"));
   },
 
   // Show/hide one item (specified via name or the item element itself).
@@ -299,17 +334,13 @@ var Tabmix = {
 
   _define: function(aType, aObj, aName, aCodeString) {
     let type = aType == "setter" ? "__defineSetter__" : "__defineGetter__";
-    let fn = new Function();
-    eval("fn = " + aCodeString);
+    let fn = eval("(" + aCodeString + ")");
     aObj[type](aName, fn);
   },
 
   toCode: function(aObj, aName, aCodeString) {
-    if (aObj) {
-      let fn = new Function();
-      eval("fn = " + aCodeString);
-      aObj[aName] = fn;
-    }
+    if (aObj)
+      aObj[aName] = eval("(" + aCodeString + ")");
     else
       eval(aName + " = " + aCodeString);
   },
@@ -317,10 +348,10 @@ var Tabmix = {
   getBoolPref: function(aPrefName, aDefault, aUseTabmixBranch) {
     var branch = aUseTabmixBranch ? "extensions.tabmix." : "";
     try {
-      return TabmixSvc.prefs.getBoolPref(branch + aPrefName);
+      return Services.prefs.getBoolPref(branch + aPrefName);
     }
     catch(er) {
-      TabmixSvc.prefs.setBoolPref(branch + aPrefName, aDefault);
+      Services.prefs.setBoolPref(branch + aPrefName, aDefault);
       return aDefault;
     }
   },
@@ -328,10 +359,10 @@ var Tabmix = {
   getIntPref: function(aPrefName, aDefault, aUseTabmixBranch) {
     var branch = aUseTabmixBranch ? "extensions.tabmix." : "";
     try {
-      return TabmixSvc.prefs.getIntPref(branch + aPrefName);
+      return Services.prefs.getIntPref(branch + aPrefName);
     }
     catch(er) {
-      TabmixSvc.prefs.setIntPref(branch + aPrefName, aDefault);
+      Services.prefs.setIntPref(branch + aPrefName, aDefault);
       return aDefault;
     }
   },
@@ -339,21 +370,30 @@ var Tabmix = {
   getCharPref: function(aPrefName, aDefault, aUseTabmixBranch) {
     var branch = aUseTabmixBranch ? "extensions.tabmix." : "";
     try {
-      return TabmixSvc.prefs.getCharPref(branch + aPrefName);
+      return Services.prefs.getCharPref(branch + aPrefName);
     }
     catch(er) {
-      TabmixSvc.prefs.setCharPref(branch + aPrefName, aDefault);
+      Services.prefs.setCharPref(branch + aPrefName, aDefault);
       return aDefault;
     }
   },
 
   getTopWin: function() {
-    return TabmixSvc.wm.getMostRecentWindow("navigator:browser");
+    return Services.wm.getMostRecentWindow("navigator:browser");
+  },
+
+  getSingleWindowMode: function TMP_getSingleWindowMode() {
+    // if we don't have any browser window opened return false
+    // so we can open new window
+    if (!this.getTopWin())
+      return false;
+    return Tabmix.prefs.getBoolPref("singleWindow");
   },
 
   lazy_import: function(aObject, aName, aModule, aSymbol, aFlag, aArg) {
     if (aFlag)
-      Tabmix[aModule + "Initialized"] = false;
+      this[aModule + "Initialized"] = false;
+    var self = this;
     XPCOMUtils.defineLazyGetter(aObject, aName, function() {
       let tmp = { };
       Components.utils.import("resource://tabmixplus/"+aModule+".jsm", tmp);
@@ -361,7 +401,7 @@ var Tabmix = {
       if ("init" in obj)
         obj.init.apply(obj, aArg);
       if (aFlag)
-        window.Tabmix[aModule + "Initialized"] = true;
+        self[aModule + "Initialized"] = true;
       return obj;
     });
   },
@@ -370,9 +410,10 @@ var Tabmix = {
     if (aOldName in aObject)
       return;
 
+    var self = this;
     XPCOMUtils.defineLazyGetter(aObject, aOldName, function() {
-      Tabmix.informAboutChangeInTabmix(aOldName, aNewName);
-      return Tabmix.getObject(aNewName);
+      self.informAboutChangeInTabmix(aOldName, aNewName);
+      return self.getObject(aNewName);
     });
   },
 
@@ -386,13 +427,13 @@ var Tabmix = {
       let index = path.indexOf("/", 2) - 3;
       let extensionName = index > -1 ?
          path.charAt(2).toUpperCase() + path.substr(3, index) + " " : "";
-      Tabmix.clog(err.message + "\n\n" + extensionName + "extension call " + aOldName +
+      this.clog(err.message + "\n\n" + extensionName + "extension call " + aOldName +
                  " from:\n" + "file: " + "chrome:" + path + "\nline: " + line
                  + "\n\nPlease inform Tabmix Plus developer"
                  + (extensionName ? ( " and " + extensionName + "developer.") : "."));
     }
     else
-      Tabmix.clog(err.message + "\n\n" + stack);
+      this.clog(err.message + "\n\n" + stack);
   },
 
   promptService: function(intParam, strParam, aWindow, aCallBack) {
@@ -403,6 +444,8 @@ var Tabmix = {
     // intParam[2] - set checkbox checked  true=1 , false=0, hide=2
     // intParam[3] - flag  - for menuList contents: flag to set menu selected item
     //                     - for textBox rename: 1 , save: 0
+///XXX temp fix
+    // intParam[4] - flag  - 1 - use Tabmix.Sessions.createMenuForDialog
 
     // we use non modal dialog when we call for prompt on startup
     // when we don't have a callBack function use modal dialog
@@ -415,8 +458,6 @@ var Tabmix = {
     for (i = 0; i < strParam.length; i++)
       dpb.SetString(i, strParam[i]);
 
-    var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"]
-                        .getService(Ci.nsIWindowWatcher);
     if (typeof(aWindow) == "undefined") {
       try { aWindow = window;
       }
@@ -424,19 +465,20 @@ var Tabmix = {
       }
     }
 
-    var dialog = ww.openWindow(aWindow,
-           "chrome://tabmixplus/content/session/promptservice.xul","",'centerscreen'+(modal ? ",modal" : ",dependent") ,dpb);
+    // we add dependent to features to make this dialog float over the window on start
+    var dialog = Services.ww.openWindow(aWindow,
+           "chrome://tabmixplus/content/session/promptservice.xul","","centerscreen" +(modal ? ",modal" : ",dependent") ,dpb);
     if (!modal)
       dialog._callBackFunction = aCallBack;
 
-    return {button: dpb.GetInt(4), checked: (dpb.GetInt(5) == TMP_CHECKBOX_CHECKED),
+    return {button: dpb.GetInt(4), checked: (dpb.GetInt(5) == this.CHECKBOX_CHECKED),
             label: dpb.GetString(5), value: dpb.GetInt(6)};
   },
 
   windowEnumerator: function Tabmix_windowEnumerator(aWindowtype) {
     if (typeof(aWindowtype) == "undefined")
       aWindowtype = "navigator:browser";
-    return TabmixSvc.wm.getEnumerator(aWindowtype);
+    return Services.wm.getEnumerator(aWindowtype);
   },
 
   numberOfWindows: function Tabmix_numberOfWindows(all, aWindowtype) {
@@ -461,22 +503,35 @@ var Tabmix = {
   JSON: {
     nsIJSON: null,
     parse: function TMP_parse(str) {
-      return "decode" in this.nsIJSON ? this.nsIJSON.decode(str) : JSON.parse(str);
+      try {
+        return JSON.parse(str);
+      } catch(ex) {
+        return "decode" in this.nsIJSON ? this.nsIJSON.decode(str) : null;
+      }
     },
     stringify: function TMP_stringify(obj) {
-      return "encode" in this.nsIJSON ? this.nsIJSON.encode(obj) : JSON.stringify(obj);
+      try {
+        return JSON.stringify(obj);
+      } catch(ex) {
+        return "encode" in this.nsIJSON ? this.nsIJSON.encode(obj) : null;
+      }
     }
   },
 
-  destroy: function() {
+  compare: function TMP_utils_compare(a, b, lessThan) {return lessThan ? a < b : a > b;},
+  itemEnd: function TMP_utils_itemEnd(item, end) {return item.boxObject.screenX + (end ? item.getBoundingClientRect().width : 0);},
+
+  destroy: function TMP_utils_destroy() {
     this._define = null;
     this.toCode = null;
-    window.removeEventListener("unload", Tabmix.destroy, false);
+    window.removeEventListener("unload", TMP_utils_destroy, false);
   }
 }
 
-Components.utils.import("resource://tabmixplus/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Services.jsm");
 Tabmix.lazy_import(window, "TabmixSvc", "Services", "TabmixSvc");
-XPCOMUtils.defineLazyServiceGetter(Tabmix.JSON, "nsIJSON", "@mozilla.org/dom/json;1", "nsIJSON");
-
+XPCOMUtils.defineLazyGetter(Tabmix.JSON, "nsIJSON", function() {
+  return Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON);
+});
 window.addEventListener("unload", Tabmix.destroy, false);
